<?php
 namespace Psy\Command; use PhpParser\Node; use PhpParser\Parser; use Psy\Context; use Psy\ContextAware; use Psy\Input\CodeArgument; use Psy\ParserFactory; use Psy\VarDumper\Presenter; use Psy\VarDumper\PresenterAware; use Symfony\Component\Console\Input\InputInterface; use Symfony\Component\Console\Input\InputOption; use Symfony\Component\Console\Output\OutputInterface; use Symfony\Component\VarDumper\Caster\Caster; class ParseCommand extends Command implements ContextAware, PresenterAware { protected $context; private $presenter; private $parserFactory; private $parsers; public function __construct($name = null) { $this->parserFactory = new ParserFactory(); $this->parsers = []; parent::__construct($name); } public function setContext(Context $context) { $this->context = $context; } public function setPresenter(Presenter $presenter) { $this->presenter = clone $presenter; $this->presenter->addCasters([ 'PhpParser\Node' => function (Node $node, array $a) { $a = [ Caster::PREFIX_VIRTUAL . 'type' => $node->getType(), Caster::PREFIX_VIRTUAL . 'attributes' => $node->getAttributes(), ]; foreach ($node->getSubNodeNames() as $name) { $a[Caster::PREFIX_VIRTUAL . $name] = $node->$name; } return $a; }, ]); } protected function configure() { $definition = [ new CodeArgument('code', CodeArgument::REQUIRED, 'PHP code to parse.'), new InputOption('depth', '', InputOption::VALUE_REQUIRED, 'Depth to parse.', 10), ]; if ($this->parserFactory->hasKindsSupport()) { $msg = 'One of PhpParser\\ParserFactory constants: ' . \implode(', ', ParserFactory::getPossibleKinds()) . " (default is based on current interpreter's version)."; $defaultKind = $this->parserFactory->getDefaultKind(); $definition[] = new InputOption('kind', '', InputOption::VALUE_REQUIRED, $msg, $defaultKind); } $this ->setName('parse') ->setDefinition($definition) ->setDescription('Parse PHP code and show the abstract syntax tree.') ->setHelp( <<<'HELP'
Parse PHP code and show the abstract syntax tree.

This command is used in the development of PsySH. Given a string of PHP code,
it pretty-prints the PHP Parser parse tree.

See https://github.com/nikic/PHP-Parser

It prolly won't be super useful for most of you, but it's here if you want to play.
HELP
); } protected function execute(InputInterface $input, OutputInterface $output) { $code = $input->getArgument('code'); if (\strpos('<?', $code) === false) { $code = '<?php ' . $code; } $parserKind = $this->parserFactory->hasKindsSupport() ? $input->getOption('kind') : null; $depth = $input->getOption('depth'); $nodes = $this->parse($this->getParser($parserKind), $code); $output->page($this->presenter->present($nodes, $depth)); $this->context->setReturnValue($nodes); } private function parse(Parser $parser, $code) { try { return $parser->parse($code); } catch (\PhpParser\Error $e) { if (\strpos($e->getMessage(), 'unexpected EOF') === false) { throw $e; } return $parser->parse($code . ';'); } } private function getParser($kind = null) { if (!\array_key_exists($kind, $this->parsers)) { $this->parsers[$kind] = $this->parserFactory->createParser($kind); } return $this->parsers[$kind]; } } 