<?php
 namespace PHPUnit\Framework; use DeepCopy\DeepCopy; use PHPUnit\Framework\Constraint\Exception as ExceptionConstraint; use PHPUnit\Framework\Constraint\ExceptionCode; use PHPUnit\Framework\Constraint\ExceptionMessage; use PHPUnit\Framework\Constraint\ExceptionMessageRegularExpression; use PHPUnit\Framework\MockObject\Generator as MockGenerator; use PHPUnit\Framework\MockObject\Matcher\AnyInvokedCount as AnyInvokedCountMatcher; use PHPUnit\Framework\MockObject\Matcher\InvokedAtIndex as InvokedAtIndexMatcher; use PHPUnit\Framework\MockObject\Matcher\InvokedAtLeastCount as InvokedAtLeastCountMatcher; use PHPUnit\Framework\MockObject\Matcher\InvokedAtLeastOnce as InvokedAtLeastOnceMatcher; use PHPUnit\Framework\MockObject\Matcher\InvokedAtMostCount as InvokedAtMostCountMatcher; use PHPUnit\Framework\MockObject\Matcher\InvokedCount as InvokedCountMatcher; use PHPUnit\Framework\MockObject\MockBuilder; use PHPUnit\Framework\MockObject\MockObject; use PHPUnit\Framework\MockObject\Stub\ConsecutiveCalls as ConsecutiveCallsStub; use PHPUnit\Framework\MockObject\Stub\Exception as ExceptionStub; use PHPUnit\Framework\MockObject\Stub\ReturnArgument as ReturnArgumentStub; use PHPUnit\Framework\MockObject\Stub\ReturnCallback as ReturnCallbackStub; use PHPUnit\Framework\MockObject\Stub\ReturnSelf as ReturnSelfStub; use PHPUnit\Framework\MockObject\Stub\ReturnStub; use PHPUnit\Framework\MockObject\Stub\ReturnValueMap as ReturnValueMapStub; use PHPUnit\Runner\BaseTestRunner; use PHPUnit\Runner\PhptTestCase; use PHPUnit\Util\GlobalState; use PHPUnit\Util\InvalidArgumentHelper; use PHPUnit\Util\PHP\AbstractPhpProcess; use Prophecy; use Prophecy\Exception\Prediction\PredictionException; use Prophecy\Prophet; use ReflectionClass; use ReflectionException; use ReflectionObject; use SebastianBergmann; use SebastianBergmann\Comparator\Comparator; use SebastianBergmann\Comparator\Factory as ComparatorFactory; use SebastianBergmann\Diff\Differ; use SebastianBergmann\Exporter\Exporter; use SebastianBergmann\GlobalState\Blacklist; use SebastianBergmann\GlobalState\Restorer; use SebastianBergmann\GlobalState\Snapshot; use SebastianBergmann\ObjectEnumerator\Enumerator; use Text_Template; use Throwable; abstract class TestCase extends Assert implements Test, SelfDescribing { protected $backupGlobals; protected $backupGlobalsBlacklist = []; protected $backupStaticAttributes; protected $backupStaticAttributesBlacklist = []; protected $runTestInSeparateProcess; private $runClassInSeparateProcess; protected $preserveGlobalState = true; private $inIsolation = false; private $data; private $dataName; private $useErrorHandler; private $expectedException; private $expectedExceptionMessage; private $expectedExceptionMessageRegExp; private $expectedExceptionCode; private $name; private $dependencies = []; private $dependencyInput = []; private $iniSettings = []; private $locale = []; private $mockObjects = []; private $mockObjectGenerator; private $status; private $statusMessage = ''; private $numAssertions = 0; private $result; private $testResult; private $output = ''; private $outputExpectedRegex; private $outputExpectedString; private $outputCallback = false; private $outputBufferingActive = false; private $outputBufferingLevel; private $snapshot; private $prophet; private $beStrictAboutChangesToGlobalState = false; private $registerMockObjectsFromTestArgumentsRecursively = false; private $warnings = []; private $groups = []; private $doesNotPerformAssertions = false; private $customComparators = []; public function __construct($name = null, array $data = [], $dataName = '') { if ($name !== null) { $this->setName($name); } $this->data = $data; $this->dataName = $dataName; } public function toString() { $class = new ReflectionClass($this); $buffer = \sprintf( '%s::%s', $class->name, $this->getName(false) ); return $buffer . $this->getDataSetAsString(); } public function count() { return 1; } public function getGroups() { return $this->groups; } public function setGroups(array $groups) { $this->groups = $groups; } public function getAnnotations() { return \PHPUnit\Util\Test::parseTestMethodAnnotations( \get_class($this), $this->name ); } public function getName($withDataSet = true) { if ($withDataSet) { return $this->name . $this->getDataSetAsString(false); } return $this->name; } public function getSize() { return \PHPUnit\Util\Test::getSize( \get_class($this), $this->getName(false) ); } public function hasSize() { return $this->getSize() !== \PHPUnit\Util\Test::UNKNOWN; } public function isSmall() { return $this->getSize() === \PHPUnit\Util\Test::SMALL; } public function isMedium() { return $this->getSize() === \PHPUnit\Util\Test::MEDIUM; } public function isLarge() { return $this->getSize() === \PHPUnit\Util\Test::LARGE; } public function getActualOutput() { if (!$this->outputBufferingActive) { return $this->output; } return \ob_get_contents(); } public function hasOutput() { if (\strlen($this->output) === 0) { return false; } if ($this->hasExpectationOnOutput()) { return false; } return true; } public function doesNotPerformAssertions() { return $this->doesNotPerformAssertions; } public function expectOutputRegex($expectedRegex) { if ($this->outputExpectedString !== null) { throw new Exception; } if (\is_string($expectedRegex) || null === $expectedRegex) { $this->outputExpectedRegex = $expectedRegex; } } public function expectOutputString($expectedString) { if ($this->outputExpectedRegex !== null) { throw new Exception; } if (\is_string($expectedString) || null === $expectedString) { $this->outputExpectedString = $expectedString; } } public function hasExpectationOnOutput() { return \is_string($this->outputExpectedString) || \is_string($this->outputExpectedRegex); } public function getExpectedException() { return $this->expectedException; } public function getExpectedExceptionCode() { return $this->expectedExceptionCode; } public function getExpectedExceptionMessage() { return $this->expectedExceptionMessage; } public function getExpectedExceptionMessageRegExp() { return $this->expectedExceptionMessageRegExp; } public function expectException($exception) { if (!\is_string($exception)) { throw InvalidArgumentHelper::factory(1, 'string'); } $this->expectedException = $exception; } public function expectExceptionCode($code) { if (!\is_int($code) && !\is_string($code)) { throw InvalidArgumentHelper::factory(1, 'integer or string'); } $this->expectedExceptionCode = $code; } public function expectExceptionMessage($message) { if (!\is_string($message)) { throw InvalidArgumentHelper::factory(1, 'string'); } $this->expectedExceptionMessage = $message; } public function expectExceptionMessageRegExp($messageRegExp) { if (!\is_string($messageRegExp)) { throw InvalidArgumentHelper::factory(1, 'string'); } $this->expectedExceptionMessageRegExp = $messageRegExp; } public function expectExceptionObject(\Exception $exception) { $this->expectException(\get_class($exception)); $this->expectExceptionMessage($exception->getMessage()); $this->expectExceptionCode($exception->getCode()); } public function setRegisterMockObjectsFromTestArgumentsRecursively($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->registerMockObjectsFromTestArgumentsRecursively = $flag; } protected function setExpectedExceptionFromAnnotation() { try { $expectedException = \PHPUnit\Util\Test::getExpectedException( \get_class($this), $this->name ); if ($expectedException !== false) { $this->expectException($expectedException['class']); if ($expectedException['code'] !== null) { $this->expectExceptionCode($expectedException['code']); } if ($expectedException['message'] !== '') { $this->expectExceptionMessage($expectedException['message']); } elseif ($expectedException['message_regex'] !== '') { $this->expectExceptionMessageRegExp($expectedException['message_regex']); } } } catch (ReflectionException $e) { } } public function setUseErrorHandler($useErrorHandler) { $this->useErrorHandler = $useErrorHandler; } protected function setUseErrorHandlerFromAnnotation() { try { $useErrorHandler = \PHPUnit\Util\Test::getErrorHandlerSettings( \get_class($this), $this->name ); if ($useErrorHandler !== null) { $this->setUseErrorHandler($useErrorHandler); } } catch (ReflectionException $e) { } } protected function checkRequirements() { if (!$this->name || !\method_exists($this, $this->name)) { return; } $missingRequirements = \PHPUnit\Util\Test::getMissingRequirements( \get_class($this), $this->name ); if (!empty($missingRequirements)) { $this->markTestSkipped(\implode(PHP_EOL, $missingRequirements)); } } public function getStatus() { return $this->status; } public function markAsRisky() { $this->status = BaseTestRunner::STATUS_RISKY; } public function getStatusMessage() { return $this->statusMessage; } public function hasFailed() { $status = $this->getStatus(); return $status == BaseTestRunner::STATUS_FAILURE || $status == BaseTestRunner::STATUS_ERROR; } public function run(TestResult $result = null) { if ($result === null) { $result = $this->createResult(); } if (!$this instanceof WarningTestCase) { $this->setTestResultObject($result); $this->setUseErrorHandlerFromAnnotation(); } if ($this->useErrorHandler !== null) { $oldErrorHandlerSetting = $result->getConvertErrorsToExceptions(); $result->convertErrorsToExceptions($this->useErrorHandler); } if (!$this instanceof WarningTestCase && !$this instanceof SkippedTestCase && !$this->handleDependencies()) { return; } $runEntireClass = $this->runClassInSeparateProcess && !$this->runTestInSeparateProcess; if (($this->runTestInSeparateProcess === true || $this->runClassInSeparateProcess === true) && $this->inIsolation !== true && !$this instanceof PhptTestCase) { $class = new ReflectionClass($this); if ($runEntireClass) { $template = new Text_Template( __DIR__ . '/../Util/PHP/Template/TestCaseClass.tpl' ); } else { $template = new Text_Template( __DIR__ . '/../Util/PHP/Template/TestCaseMethod.tpl' ); } if ($this->preserveGlobalState) { $constants = GlobalState::getConstantsAsString(); $globals = GlobalState::getGlobalsAsString(); $includedFiles = GlobalState::getIncludedFilesAsString(); $iniSettings = GlobalState::getIniSettingsAsString(); } else { $constants = ''; if (!empty($GLOBALS['__PHPUNIT_BOOTSTRAP'])) { $globals = '$GLOBALS[\'__PHPUNIT_BOOTSTRAP\'] = ' . \var_export($GLOBALS['__PHPUNIT_BOOTSTRAP'], true) . ";\n"; } else { $globals = ''; } $includedFiles = ''; $iniSettings = ''; } $coverage = $result->getCollectCodeCoverageInformation() ? 'true' : 'false'; $isStrictAboutTestsThatDoNotTestAnything = $result->isStrictAboutTestsThatDoNotTestAnything() ? 'true' : 'false'; $isStrictAboutOutputDuringTests = $result->isStrictAboutOutputDuringTests() ? 'true' : 'false'; $enforcesTimeLimit = $result->enforcesTimeLimit() ? 'true' : 'false'; $isStrictAboutTodoAnnotatedTests = $result->isStrictAboutTodoAnnotatedTests() ? 'true' : 'false'; $isStrictAboutResourceUsageDuringSmallTests = $result->isStrictAboutResourceUsageDuringSmallTests() ? 'true' : 'false'; if (\defined('PHPUNIT_COMPOSER_INSTALL')) { $composerAutoload = \var_export(PHPUNIT_COMPOSER_INSTALL, true); } else { $composerAutoload = '\'\''; } if (\defined('__PHPUNIT_PHAR__')) { $phar = \var_export(__PHPUNIT_PHAR__, true); } else { $phar = '\'\''; } if ($result->getCodeCoverage()) { $codeCoverageFilter = $result->getCodeCoverage()->filter(); } else { $codeCoverageFilter = null; } $data = \var_export(\serialize($this->data), true); $dataName = \var_export($this->dataName, true); $dependencyInput = \var_export(\serialize($this->dependencyInput), true); $includePath = \var_export(\get_include_path(), true); $codeCoverageFilter = \var_export(\serialize($codeCoverageFilter), true); $data = "'." . $data . ".'"; $dataName = "'.(" . $dataName . ").'"; $dependencyInput = "'." . $dependencyInput . ".'"; $includePath = "'." . $includePath . ".'"; $codeCoverageFilter = "'." . $codeCoverageFilter . ".'"; $configurationFilePath = $GLOBALS['__PHPUNIT_CONFIGURATION_FILE'] ?? ''; $var = [ 'composerAutoload' => $composerAutoload, 'phar' => $phar, 'filename' => $class->getFileName(), 'className' => $class->getName(), 'collectCodeCoverageInformation' => $coverage, 'data' => $data, 'dataName' => $dataName, 'dependencyInput' => $dependencyInput, 'constants' => $constants, 'globals' => $globals, 'include_path' => $includePath, 'included_files' => $includedFiles, 'iniSettings' => $iniSettings, 'isStrictAboutTestsThatDoNotTestAnything' => $isStrictAboutTestsThatDoNotTestAnything, 'isStrictAboutOutputDuringTests' => $isStrictAboutOutputDuringTests, 'enforcesTimeLimit' => $enforcesTimeLimit, 'isStrictAboutTodoAnnotatedTests' => $isStrictAboutTodoAnnotatedTests, 'isStrictAboutResourceUsageDuringSmallTests' => $isStrictAboutResourceUsageDuringSmallTests, 'codeCoverageFilter' => $codeCoverageFilter, 'configurationFilePath' => $configurationFilePath ]; if (!$runEntireClass) { $var['methodName'] = $this->name; } $template->setVar( $var ); $this->prepareTemplate($template); $php = AbstractPhpProcess::factory(); $php->runTestJob($template->render(), $this, $result); } else { $result->run($this); } if (isset($oldErrorHandlerSetting)) { $result->convertErrorsToExceptions($oldErrorHandlerSetting); } $this->result = null; return $result; } public function runBare() { $this->numAssertions = 0; $this->snapshotGlobalState(); $this->startOutputBuffering(); \clearstatcache(); $currentWorkingDirectory = \getcwd(); $hookMethods = \PHPUnit\Util\Test::getHookMethods(\get_class($this)); try { $hasMetRequirements = false; $this->checkRequirements(); $hasMetRequirements = true; if ($this->inIsolation) { foreach ($hookMethods['beforeClass'] as $method) { $this->$method(); } } $this->setExpectedExceptionFromAnnotation(); $this->setDoesNotPerformAssertionsFromAnnotation(); foreach ($hookMethods['before'] as $method) { $this->$method(); } $this->assertPreConditions(); $this->testResult = $this->runTest(); $this->verifyMockObjects(); $this->assertPostConditions(); if (!empty($this->warnings)) { throw new Warning( \implode( "\n", \array_unique($this->warnings) ) ); } $this->status = BaseTestRunner::STATUS_PASSED; } catch (IncompleteTest $e) { $this->status = BaseTestRunner::STATUS_INCOMPLETE; $this->statusMessage = $e->getMessage(); } catch (SkippedTest $e) { $this->status = BaseTestRunner::STATUS_SKIPPED; $this->statusMessage = $e->getMessage(); } catch (Warning $e) { $this->status = BaseTestRunner::STATUS_WARNING; $this->statusMessage = $e->getMessage(); } catch (AssertionFailedError $e) { $this->status = BaseTestRunner::STATUS_FAILURE; $this->statusMessage = $e->getMessage(); } catch (PredictionException $e) { $this->status = BaseTestRunner::STATUS_FAILURE; $this->statusMessage = $e->getMessage(); } catch (Throwable $_e) { $e = $_e; } $this->mockObjects = []; $this->prophet = null; try { if ($hasMetRequirements) { foreach ($hookMethods['after'] as $method) { $this->$method(); } if ($this->inIsolation) { foreach ($hookMethods['afterClass'] as $method) { $this->$method(); } } } } catch (Throwable $_e) { if (!isset($e)) { $e = $_e; } } try { $this->stopOutputBuffering(); } catch (RiskyTestError $_e) { if (!isset($e)) { $e = $_e; } } if (isset($_e)) { $this->status = BaseTestRunner::STATUS_ERROR; $this->statusMessage = $_e->getMessage(); } \clearstatcache(); if ($currentWorkingDirectory != \getcwd()) { \chdir($currentWorkingDirectory); } $this->restoreGlobalState(); $this->unregisterCustomComparators(); $this->cleanupIniSettings(); $this->cleanupLocaleSettings(); if (!isset($e)) { try { if ($this->outputExpectedRegex !== null) { $this->assertRegExp($this->outputExpectedRegex, $this->output); } elseif ($this->outputExpectedString !== null) { $this->assertEquals($this->outputExpectedString, $this->output); } } catch (Throwable $_e) { $e = $_e; } } if (isset($e)) { if ($e instanceof PredictionException) { $e = new AssertionFailedError($e->getMessage()); } $this->onNotSuccessfulTest($e); } } protected function runTest() { if ($this->name === null) { throw new Exception( 'PHPUnit\Framework\TestCase::$name must not be null.' ); } try { $class = new ReflectionClass($this); $method = $class->getMethod($this->name); } catch (ReflectionException $e) { $this->fail($e->getMessage()); } $testArguments = \array_merge($this->data, $this->dependencyInput); $this->registerMockObjectsFromTestArguments($testArguments); try { $testResult = $method->invokeArgs($this, $testArguments); } catch (Throwable $t) { $exception = $t; } if (isset($exception)) { if ($this->checkExceptionExpectations($exception)) { if ($this->expectedException !== null) { $this->assertThat( $exception, new ExceptionConstraint( $this->expectedException ) ); } if ($this->expectedExceptionMessage !== null) { $this->assertThat( $exception, new ExceptionMessage( $this->expectedExceptionMessage ) ); } if ($this->expectedExceptionMessageRegExp !== null) { $this->assertThat( $exception, new ExceptionMessageRegularExpression( $this->expectedExceptionMessageRegExp ) ); } if ($this->expectedExceptionCode !== null) { $this->assertThat( $exception, new ExceptionCode( $this->expectedExceptionCode ) ); } return; } throw $exception; } if ($this->expectedException !== null) { $this->assertThat( null, new ExceptionConstraint( $this->expectedException ) ); } elseif ($this->expectedExceptionMessage !== null) { $this->numAssertions++; throw new AssertionFailedError( \sprintf( 'Failed asserting that exception with message "%s" is thrown', $this->expectedExceptionMessage ) ); } elseif ($this->expectedExceptionMessageRegExp !== null) { $this->numAssertions++; throw new AssertionFailedError( \sprintf( 'Failed asserting that exception with message matching "%s" is thrown', $this->expectedExceptionMessageRegExp ) ); } elseif ($this->expectedExceptionCode !== null) { $this->numAssertions++; throw new AssertionFailedError( \sprintf( 'Failed asserting that exception with code "%s" is thrown', $this->expectedExceptionCode ) ); } return $testResult; } protected function verifyMockObjects() { foreach ($this->mockObjects as $mockObject) { if ($mockObject->__phpunit_hasMatchers()) { $this->numAssertions++; } $mockObject->__phpunit_verify( $this->shouldInvocationMockerBeReset($mockObject) ); } if ($this->prophet !== null) { try { $this->prophet->checkPredictions(); } catch (Throwable $t) { } foreach ($this->prophet->getProphecies() as $objectProphecy) { foreach ($objectProphecy->getMethodProphecies() as $methodProphecies) { foreach ($methodProphecies as $methodProphecy) { $this->numAssertions += \count($methodProphecy->getCheckedPredictions()); } } } if (isset($t)) { throw $t; } } } public function setName($name) { $this->name = $name; } public function setDependencies(array $dependencies) { $this->dependencies = $dependencies; } public function hasDependencies() { return \count($this->dependencies) > 0; } public function setDependencyInput(array $dependencyInput) { $this->dependencyInput = $dependencyInput; } public function setBeStrictAboutChangesToGlobalState($beStrictAboutChangesToGlobalState) { $this->beStrictAboutChangesToGlobalState = $beStrictAboutChangesToGlobalState; } public function setBackupGlobals($backupGlobals) { if (null === $this->backupGlobals && \is_bool($backupGlobals)) { $this->backupGlobals = $backupGlobals; } } public function setBackupStaticAttributes($backupStaticAttributes) { if (null === $this->backupStaticAttributes && \is_bool($backupStaticAttributes)) { $this->backupStaticAttributes = $backupStaticAttributes; } } public function setRunTestInSeparateProcess($runTestInSeparateProcess) { if (\is_bool($runTestInSeparateProcess)) { if ($this->runTestInSeparateProcess === null) { $this->runTestInSeparateProcess = $runTestInSeparateProcess; } } else { throw InvalidArgumentHelper::factory(1, 'boolean'); } } public function setRunClassInSeparateProcess($runClassInSeparateProcess) { if (\is_bool($runClassInSeparateProcess)) { if ($this->runClassInSeparateProcess === null) { $this->runClassInSeparateProcess = $runClassInSeparateProcess; } } else { throw InvalidArgumentHelper::factory(1, 'boolean'); } } public function setPreserveGlobalState($preserveGlobalState) { if (\is_bool($preserveGlobalState)) { $this->preserveGlobalState = $preserveGlobalState; } else { throw InvalidArgumentHelper::factory(1, 'boolean'); } } public function setInIsolation($inIsolation) { if (\is_bool($inIsolation)) { $this->inIsolation = $inIsolation; } else { throw InvalidArgumentHelper::factory(1, 'boolean'); } } public function isInIsolation() { return $this->inIsolation; } public function getResult() { return $this->testResult; } public function setResult($result) { $this->testResult = $result; } public function setOutputCallback($callback) { if (!\is_callable($callback)) { throw InvalidArgumentHelper::factory(1, 'callback'); } $this->outputCallback = $callback; } public function getTestResultObject() { return $this->result; } public function setTestResultObject(TestResult $result) { $this->result = $result; } public function registerMockObject(MockObject $mockObject) { $this->mockObjects[] = $mockObject; } protected function iniSet($varName, $newValue) { if (!\is_string($varName)) { throw InvalidArgumentHelper::factory(1, 'string'); } $currentValue = \ini_set($varName, $newValue); if ($currentValue !== false) { $this->iniSettings[$varName] = $currentValue; } else { throw new Exception( \sprintf( 'INI setting "%s" could not be set to "%s".', $varName, $newValue ) ); } } protected function setLocale() { $args = \func_get_args(); if (\count($args) < 2) { throw new Exception; } list($category, $locale) = $args; $categories = [ LC_ALL, LC_COLLATE, LC_CTYPE, LC_MONETARY, LC_NUMERIC, LC_TIME ]; if (\defined('LC_MESSAGES')) { $categories[] = LC_MESSAGES; } if (!\in_array($category, $categories)) { throw new Exception; } if (!\is_array($locale) && !\is_string($locale)) { throw new Exception; } $this->locale[$category] = \setlocale($category, 0); $result = \call_user_func_array('setlocale', $args); if ($result === false) { throw new Exception( 'The locale functionality is not implemented on your platform, ' . 'the specified locale does not exist or the category name is ' . 'invalid.' ); } } public function getMockBuilder($className) { return new MockBuilder($this, $className); } protected function createMock($originalClassName) { return $this->getMockBuilder($originalClassName) ->disableOriginalConstructor() ->disableOriginalClone() ->disableArgumentCloning() ->disallowMockingUnknownTypes() ->getMock(); } protected function createConfiguredMock($originalClassName, array $configuration) { $o = $this->createMock($originalClassName); foreach ($configuration as $method => $return) { $o->method($method)->willReturn($return); } return $o; } protected function createPartialMock($originalClassName, array $methods) { return $this->getMockBuilder($originalClassName) ->disableOriginalConstructor() ->disableOriginalClone() ->disableArgumentCloning() ->disallowMockingUnknownTypes() ->setMethods(empty($methods) ? null : $methods) ->getMock(); } protected function createTestProxy($originalClassName, array $constructorArguments = []) { return $this->getMockBuilder($originalClassName) ->setConstructorArgs($constructorArguments) ->enableProxyingToOriginalMethods() ->getMock(); } protected function getMockClass($originalClassName, $methods = [], array $arguments = [], $mockClassName = '', $callOriginalConstructor = false, $callOriginalClone = true, $callAutoload = true, $cloneArguments = false) { $mock = $this->getMockObjectGenerator()->getMock( $originalClassName, $methods, $arguments, $mockClassName, $callOriginalConstructor, $callOriginalClone, $callAutoload, $cloneArguments ); return \get_class($mock); } protected function getMockForAbstractClass($originalClassName, array $arguments = [], $mockClassName = '', $callOriginalConstructor = true, $callOriginalClone = true, $callAutoload = true, $mockedMethods = [], $cloneArguments = false) { $mockObject = $this->getMockObjectGenerator()->getMockForAbstractClass( $originalClassName, $arguments, $mockClassName, $callOriginalConstructor, $callOriginalClone, $callAutoload, $mockedMethods, $cloneArguments ); $this->registerMockObject($mockObject); return $mockObject; } protected function getMockFromWsdl($wsdlFile, $originalClassName = '', $mockClassName = '', array $methods = [], $callOriginalConstructor = true, array $options = []) { if ($originalClassName === '') { $originalClassName = \pathinfo(\basename(\parse_url($wsdlFile)['path']), PATHINFO_FILENAME); } if (!\class_exists($originalClassName)) { eval( $this->getMockObjectGenerator()->generateClassFromWsdl( $wsdlFile, $originalClassName, $methods, $options ) ); } $mockObject = $this->getMockObjectGenerator()->getMock( $originalClassName, $methods, ['', $options], $mockClassName, $callOriginalConstructor, false, false ); $this->registerMockObject($mockObject); return $mockObject; } protected function getMockForTrait($traitName, array $arguments = [], $mockClassName = '', $callOriginalConstructor = true, $callOriginalClone = true, $callAutoload = true, $mockedMethods = [], $cloneArguments = false) { $mockObject = $this->getMockObjectGenerator()->getMockForTrait( $traitName, $arguments, $mockClassName, $callOriginalConstructor, $callOriginalClone, $callAutoload, $mockedMethods, $cloneArguments ); $this->registerMockObject($mockObject); return $mockObject; } protected function getObjectForTrait($traitName, array $arguments = [], $traitClassName = '', $callOriginalConstructor = true, $callOriginalClone = true, $callAutoload = true) { return $this->getMockObjectGenerator()->getObjectForTrait( $traitName, $arguments, $traitClassName, $callOriginalConstructor, $callOriginalClone, $callAutoload ); } protected function prophesize($classOrInterface = null) { return $this->getProphet()->prophesize($classOrInterface); } public function addToAssertionCount($count) { $this->numAssertions += $count; } public function getNumAssertions() { return $this->numAssertions; } public static function any() { return new AnyInvokedCountMatcher; } public static function never() { return new InvokedCountMatcher(0); } public static function atLeast($requiredInvocations) { return new InvokedAtLeastCountMatcher( $requiredInvocations ); } public static function atLeastOnce() { return new InvokedAtLeastOnceMatcher; } public static function once() { return new InvokedCountMatcher(1); } public static function exactly($count) { return new InvokedCountMatcher($count); } public static function atMost($allowedInvocations) { return new InvokedAtMostCountMatcher($allowedInvocations); } public static function at($index) { return new InvokedAtIndexMatcher($index); } public static function returnValue($value) { return new ReturnStub($value); } public static function returnValueMap(array $valueMap) { return new ReturnValueMapStub($valueMap); } public static function returnArgument($argumentIndex) { return new ReturnArgumentStub($argumentIndex); } public static function returnCallback($callback) { return new ReturnCallbackStub($callback); } public static function returnSelf() { return new ReturnSelfStub; } public static function throwException(Throwable $exception) { return new ExceptionStub($exception); } public static function onConsecutiveCalls() { $args = \func_get_args(); return new ConsecutiveCallsStub($args); } public function usesDataProvider() { return !empty($this->data); } public function dataDescription() { return \is_string($this->dataName) ? $this->dataName : ''; } public function dataName() { return $this->dataName; } public function registerComparator(Comparator $comparator) { ComparatorFactory::getInstance()->register($comparator); $this->customComparators[] = $comparator; } public function getDataSetAsString($includeData = true) { $buffer = ''; if (!empty($this->data)) { if (\is_int($this->dataName)) { $buffer .= \sprintf(' with data set #%d', $this->dataName); } else { $buffer .= \sprintf(' with data set "%s"', $this->dataName); } $exporter = new Exporter; if ($includeData) { $buffer .= \sprintf(' (%s)', $exporter->shortenedRecursiveExport($this->data)); } } return $buffer; } protected function getProvidedData() { return $this->data; } protected function createResult() { return new TestResult; } protected function handleDependencies() { if (!empty($this->dependencies) && !$this->inIsolation) { $className = \get_class($this); $passed = $this->result->passed(); $passedKeys = \array_keys($passed); $numKeys = \count($passedKeys); for ($i = 0; $i < $numKeys; $i++) { $pos = \strpos($passedKeys[$i], ' with data set'); if ($pos !== false) { $passedKeys[$i] = \substr($passedKeys[$i], 0, $pos); } } $passedKeys = \array_flip(\array_unique($passedKeys)); foreach ($this->dependencies as $dependency) { $deepClone = false; $shallowClone = false; if (\strpos($dependency, 'clone ') === 0) { $deepClone = true; $dependency = \substr($dependency, \strlen('clone ')); } elseif (\strpos($dependency, '!clone ') === 0) { $deepClone = false; $dependency = \substr($dependency, \strlen('!clone ')); } if (\strpos($dependency, 'shallowClone ') === 0) { $shallowClone = true; $dependency = \substr($dependency, \strlen('shallowClone ')); } elseif (\strpos($dependency, '!shallowClone ') === 0) { $shallowClone = false; $dependency = \substr($dependency, \strlen('!shallowClone ')); } if (\strpos($dependency, '::') === false) { $dependency = $className . '::' . $dependency; } if (!isset($passedKeys[$dependency])) { $this->result->startTest($this); $this->result->addError( $this, new SkippedTestError( \sprintf( 'This test depends on "%s" to pass.', $dependency ) ), 0 ); $this->result->endTest($this, 0); return false; } if (isset($passed[$dependency])) { if ($passed[$dependency]['size'] != \PHPUnit\Util\Test::UNKNOWN && $this->getSize() != \PHPUnit\Util\Test::UNKNOWN && $passed[$dependency]['size'] > $this->getSize()) { $this->result->addError( $this, new SkippedTestError( 'This test depends on a test that is larger than itself.' ), 0 ); return false; } if ($deepClone) { $deepCopy = new DeepCopy; $deepCopy->skipUncloneable(false); $this->dependencyInput[$dependency] = $deepCopy->copy($passed[$dependency]['result']); } elseif ($shallowClone) { $this->dependencyInput[$dependency] = clone $passed[$dependency]['result']; } else { $this->dependencyInput[$dependency] = $passed[$dependency]['result']; } } else { $this->dependencyInput[$dependency] = null; } } } return true; } public static function setUpBeforeClass() { } protected function setUp() { } protected function assertPreConditions() { } protected function assertPostConditions() { } protected function tearDown() { } public static function tearDownAfterClass() { } protected function onNotSuccessfulTest(Throwable $t) { throw $t; } protected function prepareTemplate(Text_Template $template) { } private function getMockObjectGenerator() { if (null === $this->mockObjectGenerator) { $this->mockObjectGenerator = new MockGenerator; } return $this->mockObjectGenerator; } private function startOutputBuffering() { \ob_start(); $this->outputBufferingActive = true; $this->outputBufferingLevel = \ob_get_level(); } private function stopOutputBuffering() { if (\ob_get_level() != $this->outputBufferingLevel) { while (\ob_get_level() >= $this->outputBufferingLevel) { \ob_end_clean(); } throw new RiskyTestError( 'Test code or tested code did not (only) close its own output buffers' ); } $output = \ob_get_contents(); if ($this->outputCallback === false) { $this->output = $output; } else { $this->output = \call_user_func_array( $this->outputCallback, [$output] ); } \ob_end_clean(); $this->outputBufferingActive = false; $this->outputBufferingLevel = \ob_get_level(); } private function snapshotGlobalState() { if ($this->runTestInSeparateProcess || $this->inIsolation || (!$this->backupGlobals === true && !$this->backupStaticAttributes)) { return; } $this->snapshot = $this->createGlobalStateSnapshot($this->backupGlobals === true); } private function restoreGlobalState() { if (!$this->snapshot instanceof Snapshot) { return; } if ($this->beStrictAboutChangesToGlobalState) { try { $this->compareGlobalStateSnapshots( $this->snapshot, $this->createGlobalStateSnapshot($this->backupGlobals === true) ); } catch (RiskyTestError $rte) { } } $restorer = new Restorer; if ($this->backupGlobals === true) { $restorer->restoreGlobalVariables($this->snapshot); } if ($this->backupStaticAttributes) { $restorer->restoreStaticAttributes($this->snapshot); } $this->snapshot = null; if (isset($rte)) { throw $rte; } } private function createGlobalStateSnapshot($backupGlobals) { $blacklist = new Blacklist; foreach ($this->backupGlobalsBlacklist as $globalVariable) { $blacklist->addGlobalVariable($globalVariable); } if (!\defined('PHPUNIT_TESTSUITE')) { $blacklist->addClassNamePrefix('PHPUnit'); $blacklist->addClassNamePrefix('File_Iterator'); $blacklist->addClassNamePrefix('SebastianBergmann\CodeCoverage'); $blacklist->addClassNamePrefix('PHP_Invoker'); $blacklist->addClassNamePrefix('PHP_Timer'); $blacklist->addClassNamePrefix('PHP_Token'); $blacklist->addClassNamePrefix('Symfony'); $blacklist->addClassNamePrefix('Text_Template'); $blacklist->addClassNamePrefix('Doctrine\Instantiator'); $blacklist->addClassNamePrefix('Prophecy'); foreach ($this->backupStaticAttributesBlacklist as $class => $attributes) { foreach ($attributes as $attribute) { $blacklist->addStaticAttribute($class, $attribute); } } } return new Snapshot( $blacklist, $backupGlobals, (bool) $this->backupStaticAttributes, false, false, false, false, false, false, false ); } private function compareGlobalStateSnapshots(Snapshot $before, Snapshot $after) { $backupGlobals = $this->backupGlobals === null || $this->backupGlobals === true; if ($backupGlobals) { $this->compareGlobalStateSnapshotPart( $before->globalVariables(), $after->globalVariables(), "--- Global variables before the test\n+++ Global variables after the test\n" ); $this->compareGlobalStateSnapshotPart( $before->superGlobalVariables(), $after->superGlobalVariables(), "--- Super-global variables before the test\n+++ Super-global variables after the test\n" ); } if ($this->backupStaticAttributes) { $this->compareGlobalStateSnapshotPart( $before->staticAttributes(), $after->staticAttributes(), "--- Static attributes before the test\n+++ Static attributes after the test\n" ); } } private function compareGlobalStateSnapshotPart(array $before, array $after, $header) { if ($before != $after) { $differ = new Differ($header); $exporter = new Exporter; $diff = $differ->diff( $exporter->export($before), $exporter->export($after) ); throw new RiskyTestError( $diff ); } } private function getProphet() { if ($this->prophet === null) { $this->prophet = new Prophet; } return $this->prophet; } private function shouldInvocationMockerBeReset(MockObject $mock) { $enumerator = new Enumerator; foreach ($enumerator->enumerate($this->dependencyInput) as $object) { if ($mock === $object) { return false; } } if (!\is_array($this->testResult) && !\is_object($this->testResult)) { return true; } foreach ($enumerator->enumerate($this->testResult) as $object) { if ($mock === $object) { return false; } } return true; } private function registerMockObjectsFromTestArguments(array $testArguments, array &$visited = []) { if ($this->registerMockObjectsFromTestArgumentsRecursively) { $enumerator = new Enumerator; foreach ($enumerator->enumerate($testArguments) as $object) { if ($object instanceof MockObject) { $this->registerMockObject($object); } } } else { foreach ($testArguments as $testArgument) { if ($testArgument instanceof MockObject) { if ($this->isCloneable($testArgument)) { $testArgument = clone $testArgument; } $this->registerMockObject($testArgument); } elseif (\is_array($testArgument) && !\in_array($testArgument, $visited, true)) { $visited[] = $testArgument; $this->registerMockObjectsFromTestArguments( $testArgument, $visited ); } } } } private function setDoesNotPerformAssertionsFromAnnotation() { $annotations = $this->getAnnotations(); if (isset($annotations['method']['doesNotPerformAssertions'])) { $this->doesNotPerformAssertions = true; } } private function isCloneable(MockObject $testArgument) { $reflector = new ReflectionObject($testArgument); if (!$reflector->isCloneable()) { return false; } if ($reflector->hasMethod('__clone') && $reflector->getMethod('__clone')->isPublic()) { return true; } return false; } private function unregisterCustomComparators() { $factory = ComparatorFactory::getInstance(); foreach ($this->customComparators as $comparator) { $factory->unregister($comparator); } $this->customComparators = []; } private function cleanupIniSettings() { foreach ($this->iniSettings as $varName => $oldValue) { \ini_set($varName, $oldValue); } $this->iniSettings = []; } private function cleanupLocaleSettings() { foreach ($this->locale as $category => $locale) { \setlocale($category, $locale); } $this->locale = []; } private function checkExceptionExpectations(Throwable $throwable): bool { $result = false; if ($this->expectedException !== null || $this->expectedExceptionCode !== null || $this->expectedExceptionMessage !== null || $this->expectedExceptionMessageRegExp !== null) { $result = true; } if ($throwable instanceof Exception) { $result = false; } if (\is_string($this->expectedException)) { $reflector = new ReflectionClass($this->expectedException); if ($this->expectedException === 'PHPUnit\Framework\Exception' || $this->expectedException === '\PHPUnit\Framework\Exception' || $reflector->isSubclassOf('PHPUnit\Framework\Exception')) { $result = true; } } return $result; } } 