<?php
 namespace PHPUnit\Framework; use AssertionError; use Countable; use Error; use PHP_Invoker; use PHP_Invoker_TimeoutException; use PHP_Timer; use PHPUnit\Framework\MockObject\Exception as MockObjectException; use PHPUnit\Util\Blacklist; use PHPUnit\Util\InvalidArgumentHelper; use PHPUnit\Util\Printer; use ReflectionClass; use SebastianBergmann\CodeCoverage\CodeCoverage; use SebastianBergmann\CodeCoverage\CoveredCodeNotExecutedException as OriginalCoveredCodeNotExecutedException; use SebastianBergmann\CodeCoverage\Exception as OriginalCodeCoverageException; use SebastianBergmann\CodeCoverage\MissingCoversAnnotationException as OriginalMissingCoversAnnotationException; use SebastianBergmann\CodeCoverage\UnintentionallyCoveredCodeException; use SebastianBergmann\ResourceOperations\ResourceOperations; use Throwable; class TestResult implements Countable { protected $passed = []; protected $errors = []; protected $failures = []; protected $warnings = []; protected $notImplemented = []; protected $risky = []; protected $skipped = []; protected $listeners = []; protected $runTests = 0; protected $time = 0; protected $topTestSuite; protected $codeCoverage; protected $convertErrorsToExceptions = true; protected $stop = false; protected $stopOnError = false; protected $stopOnFailure = false; protected $stopOnWarning = false; protected $beStrictAboutTestsThatDoNotTestAnything = true; protected $beStrictAboutOutputDuringTests = false; protected $beStrictAboutTodoAnnotatedTests = false; protected $beStrictAboutResourceUsageDuringSmallTests = false; protected $enforceTimeLimit = false; protected $timeoutForSmallTests = 1; protected $timeoutForMediumTests = 10; protected $timeoutForLargeTests = 60; protected $stopOnRisky = false; protected $stopOnIncomplete = false; protected $stopOnSkipped = false; protected $lastTestFailed = false; private $registerMockObjectsFromTestArgumentsRecursively = false; public function addListener(TestListener $listener) { $this->listeners[] = $listener; } public function removeListener(TestListener $listener) { foreach ($this->listeners as $key => $_listener) { if ($listener === $_listener) { unset($this->listeners[$key]); } } } public function flushListeners() { foreach ($this->listeners as $listener) { if ($listener instanceof Printer) { $listener->flush(); } } } public function addError(Test $test, Throwable $t, $time) { if ($t instanceof RiskyTest) { $this->risky[] = new TestFailure($test, $t); $notifyMethod = 'addRiskyTest'; if ($test instanceof TestCase) { $test->markAsRisky(); } if ($this->stopOnRisky) { $this->stop(); } } elseif ($t instanceof IncompleteTest) { $this->notImplemented[] = new TestFailure($test, $t); $notifyMethod = 'addIncompleteTest'; if ($this->stopOnIncomplete) { $this->stop(); } } elseif ($t instanceof SkippedTest) { $this->skipped[] = new TestFailure($test, $t); $notifyMethod = 'addSkippedTest'; if ($this->stopOnSkipped) { $this->stop(); } } else { $this->errors[] = new TestFailure($test, $t); $notifyMethod = 'addError'; if ($this->stopOnError || $this->stopOnFailure) { $this->stop(); } } if ($t instanceof Error) { $t = new ExceptionWrapper($t); } foreach ($this->listeners as $listener) { $listener->$notifyMethod($test, $t, $time); } $this->lastTestFailed = true; $this->time += $time; } public function addWarning(Test $test, Warning $e, $time) { if ($this->stopOnWarning) { $this->stop(); } $this->warnings[] = new TestFailure($test, $e); foreach ($this->listeners as $listener) { $listener->addWarning($test, $e, $time); } $this->time += $time; } public function addFailure(Test $test, AssertionFailedError $e, $time) { if ($e instanceof RiskyTest || $e instanceof OutputError) { $this->risky[] = new TestFailure($test, $e); $notifyMethod = 'addRiskyTest'; if ($test instanceof TestCase) { $test->markAsRisky(); } if ($this->stopOnRisky) { $this->stop(); } } elseif ($e instanceof IncompleteTest) { $this->notImplemented[] = new TestFailure($test, $e); $notifyMethod = 'addIncompleteTest'; if ($this->stopOnIncomplete) { $this->stop(); } } elseif ($e instanceof SkippedTest) { $this->skipped[] = new TestFailure($test, $e); $notifyMethod = 'addSkippedTest'; if ($this->stopOnSkipped) { $this->stop(); } } else { $this->failures[] = new TestFailure($test, $e); $notifyMethod = 'addFailure'; if ($this->stopOnFailure) { $this->stop(); } } foreach ($this->listeners as $listener) { $listener->$notifyMethod($test, $e, $time); } $this->lastTestFailed = true; $this->time += $time; } public function startTestSuite(TestSuite $suite) { if ($this->topTestSuite === null) { $this->topTestSuite = $suite; } foreach ($this->listeners as $listener) { $listener->startTestSuite($suite); } } public function endTestSuite(TestSuite $suite) { foreach ($this->listeners as $listener) { $listener->endTestSuite($suite); } } public function startTest(Test $test) { $this->lastTestFailed = false; $this->runTests += \count($test); foreach ($this->listeners as $listener) { $listener->startTest($test); } } public function endTest(Test $test, $time) { foreach ($this->listeners as $listener) { $listener->endTest($test, $time); } if (!$this->lastTestFailed && $test instanceof TestCase) { $class = \get_class($test); $key = $class . '::' . $test->getName(); $this->passed[$key] = [ 'result' => $test->getResult(), 'size' => \PHPUnit\Util\Test::getSize( $class, $test->getName(false) ) ]; $this->time += $time; } } public function allHarmless() { return $this->riskyCount() == 0; } public function riskyCount() { return \count($this->risky); } public function allCompletelyImplemented() { return $this->notImplementedCount() == 0; } public function notImplementedCount() { return \count($this->notImplemented); } public function risky() { return $this->risky; } public function notImplemented() { return $this->notImplemented; } public function noneSkipped() { return $this->skippedCount() == 0; } public function skippedCount() { return \count($this->skipped); } public function skipped() { return $this->skipped; } public function errorCount() { return \count($this->errors); } public function errors() { return $this->errors; } public function failureCount() { return \count($this->failures); } public function failures() { return $this->failures; } public function warningCount() { return \count($this->warnings); } public function warnings() { return $this->warnings; } public function passed() { return $this->passed; } public function topTestSuite() { return $this->topTestSuite; } public function getCollectCodeCoverageInformation() { return $this->codeCoverage !== null; } public function run(Test $test) { Assert::resetCount(); $coversNothing = false; if ($test instanceof TestCase) { $test->setRegisterMockObjectsFromTestArgumentsRecursively( $this->registerMockObjectsFromTestArgumentsRecursively ); $annotations = $test->getAnnotations(); if (isset($annotations['class']['coversNothing']) || isset($annotations['method']['coversNothing'])) { $coversNothing = true; } } $error = false; $failure = false; $warning = false; $incomplete = false; $risky = false; $skipped = false; $this->startTest($test); $errorHandlerSet = false; if ($this->convertErrorsToExceptions) { $oldErrorHandler = \set_error_handler( [\PHPUnit\Util\ErrorHandler::class, 'handleError'], E_ALL | E_STRICT ); if ($oldErrorHandler === null) { $errorHandlerSet = true; } else { \restore_error_handler(); } } $collectCodeCoverage = $this->codeCoverage !== null && !$test instanceof WarningTestCase && !$coversNothing; if ($collectCodeCoverage) { $this->codeCoverage->start($test); } $monitorFunctions = $this->beStrictAboutResourceUsageDuringSmallTests && !$test instanceof WarningTestCase && $test->getSize() == \PHPUnit\Util\Test::SMALL && \function_exists('xdebug_start_function_monitor'); if ($monitorFunctions) { \xdebug_start_function_monitor(ResourceOperations::getFunctions()); } PHP_Timer::start(); try { if (!$test instanceof WarningTestCase && $test->getSize() != \PHPUnit\Util\Test::UNKNOWN && $this->enforceTimeLimit && \extension_loaded('pcntl') && \class_exists('PHP_Invoker')) { switch ($test->getSize()) { case \PHPUnit\Util\Test::SMALL: $_timeout = $this->timeoutForSmallTests; break; case \PHPUnit\Util\Test::MEDIUM: $_timeout = $this->timeoutForMediumTests; break; case \PHPUnit\Util\Test::LARGE: $_timeout = $this->timeoutForLargeTests; break; } $invoker = new PHP_Invoker; $invoker->invoke([$test, 'runBare'], [], $_timeout); } else { $test->runBare(); } } catch (PHP_Invoker_TimeoutException $e) { $this->addFailure( $test, new RiskyTestError( $e->getMessage() ), $_timeout ); $risky = true; } catch (MockObjectException $e) { $e = new Warning( $e->getMessage() ); $warning = true; } catch (AssertionFailedError $e) { $failure = true; if ($e instanceof RiskyTestError) { $risky = true; } elseif ($e instanceof IncompleteTestError) { $incomplete = true; } elseif ($e instanceof SkippedTestError) { $skipped = true; } } catch (AssertionError $e) { $test->addToAssertionCount(1); $failure = true; $frame = $e->getTrace()[0]; $e = new AssertionFailedError( \sprintf( '%s in %s:%s', $e->getMessage(), $frame['file'], $frame['line'] ) ); } catch (Warning $e) { $warning = true; } catch (Exception $e) { $error = true; } catch (Throwable $e) { $e = new ExceptionWrapper($e); $error = true; } $time = PHP_Timer::stop(); $test->addToAssertionCount(Assert::getCount()); if ($monitorFunctions) { $blacklist = new Blacklist; $functions = \xdebug_get_monitored_functions(); \xdebug_stop_function_monitor(); foreach ($functions as $function) { if (!$blacklist->isBlacklisted($function['filename'])) { $this->addFailure( $test, new RiskyTestError( \sprintf( '%s() used in %s:%s', $function['function'], $function['filename'], $function['lineno'] ) ), $time ); } } } if ($this->beStrictAboutTestsThatDoNotTestAnything && $test->getNumAssertions() == 0) { $risky = true; } if ($collectCodeCoverage) { $append = !$risky && !$incomplete && !$skipped; $linesToBeCovered = []; $linesToBeUsed = []; if ($append && $test instanceof TestCase) { try { $linesToBeCovered = \PHPUnit\Util\Test::getLinesToBeCovered( \get_class($test), $test->getName(false) ); $linesToBeUsed = \PHPUnit\Util\Test::getLinesToBeUsed( \get_class($test), $test->getName(false) ); } catch (InvalidCoversTargetException $cce) { $this->addWarning( $test, new Warning( $cce->getMessage() ), $time ); } } try { $this->codeCoverage->stop( $append, $linesToBeCovered, $linesToBeUsed ); } catch (UnintentionallyCoveredCodeException $cce) { $this->addFailure( $test, new UnintentionallyCoveredCodeError( 'This test executed code that is not listed as code to be covered or used:' . PHP_EOL . $cce->getMessage() ), $time ); } catch (OriginalCoveredCodeNotExecutedException $cce) { $this->addFailure( $test, new CoveredCodeNotExecutedException( 'This test did not execute all the code that is listed as code to be covered:' . PHP_EOL . $cce->getMessage() ), $time ); } catch (OriginalMissingCoversAnnotationException $cce) { if ($linesToBeCovered !== false) { $this->addFailure( $test, new MissingCoversAnnotationException( 'This test does not have a @covers annotation but is expected to have one' ), $time ); } } catch (OriginalCodeCoverageException $cce) { $error = true; if (!isset($e)) { $e = $cce; } } } if ($errorHandlerSet === true) { \restore_error_handler(); } if ($error === true) { $this->addError($test, $e, $time); } elseif ($failure === true) { $this->addFailure($test, $e, $time); } elseif ($warning === true) { $this->addWarning($test, $e, $time); } elseif ($this->beStrictAboutTestsThatDoNotTestAnything && !$test->doesNotPerformAssertions() && $test->getNumAssertions() == 0) { $this->addFailure( $test, new RiskyTestError( 'This test did not perform any assertions' ), $time ); } elseif ($this->beStrictAboutOutputDuringTests && $test->hasOutput()) { $this->addFailure( $test, new OutputError( \sprintf( 'This test printed output: %s', $test->getActualOutput() ) ), $time ); } elseif ($this->beStrictAboutTodoAnnotatedTests && $test instanceof TestCase) { $annotations = $test->getAnnotations(); if (isset($annotations['method']['todo'])) { $this->addFailure( $test, new RiskyTestError( 'Test method is annotated with @todo' ), $time ); } } $this->endTest($test, $time); } public function count() { return $this->runTests; } public function shouldStop() { return $this->stop; } public function stop() { $this->stop = true; } public function getCodeCoverage() { return $this->codeCoverage; } public function setCodeCoverage(CodeCoverage $codeCoverage) { $this->codeCoverage = $codeCoverage; } public function convertErrorsToExceptions($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->convertErrorsToExceptions = $flag; } public function getConvertErrorsToExceptions() { return $this->convertErrorsToExceptions; } public function stopOnError($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->stopOnError = $flag; } public function stopOnFailure($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->stopOnFailure = $flag; } public function stopOnWarning($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->stopOnWarning = $flag; } public function beStrictAboutTestsThatDoNotTestAnything($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->beStrictAboutTestsThatDoNotTestAnything = $flag; } public function isStrictAboutTestsThatDoNotTestAnything() { return $this->beStrictAboutTestsThatDoNotTestAnything; } public function beStrictAboutOutputDuringTests($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->beStrictAboutOutputDuringTests = $flag; } public function isStrictAboutOutputDuringTests() { return $this->beStrictAboutOutputDuringTests; } public function beStrictAboutResourceUsageDuringSmallTests($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->beStrictAboutResourceUsageDuringSmallTests = $flag; } public function isStrictAboutResourceUsageDuringSmallTests() { return $this->beStrictAboutResourceUsageDuringSmallTests; } public function enforceTimeLimit($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->enforceTimeLimit = $flag; } public function enforcesTimeLimit() { return $this->enforceTimeLimit; } public function beStrictAboutTodoAnnotatedTests($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->beStrictAboutTodoAnnotatedTests = $flag; } public function isStrictAboutTodoAnnotatedTests() { return $this->beStrictAboutTodoAnnotatedTests; } public function stopOnRisky($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->stopOnRisky = $flag; } public function stopOnIncomplete($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->stopOnIncomplete = $flag; } public function stopOnSkipped($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->stopOnSkipped = $flag; } public function time() { return $this->time; } public function wasSuccessful() { return empty($this->errors) && empty($this->failures) && empty($this->warnings); } public function setTimeoutForSmallTests($timeout) { if (!\is_int($timeout)) { throw InvalidArgumentHelper::factory(1, 'integer'); } $this->timeoutForSmallTests = $timeout; } public function setTimeoutForMediumTests($timeout) { if (!\is_int($timeout)) { throw InvalidArgumentHelper::factory(1, 'integer'); } $this->timeoutForMediumTests = $timeout; } public function setTimeoutForLargeTests($timeout) { if (!\is_int($timeout)) { throw InvalidArgumentHelper::factory(1, 'integer'); } $this->timeoutForLargeTests = $timeout; } public function getTimeoutForLargeTests() { return $this->timeoutForLargeTests; } public function setRegisterMockObjectsFromTestArgumentsRecursively($flag) { if (!\is_bool($flag)) { throw InvalidArgumentHelper::factory(1, 'boolean'); } $this->registerMockObjectsFromTestArgumentsRecursively = $flag; } protected function getHierarchy($className, $asReflectionObjects = false) { if ($asReflectionObjects) { $classes = [new ReflectionClass($className)]; } else { $classes = [$className]; } $done = false; while (!$done) { if ($asReflectionObjects) { $class = new ReflectionClass( $classes[\count($classes) - 1]->getName() ); } else { $class = new ReflectionClass($classes[\count($classes) - 1]); } $parent = $class->getParentClass(); if ($parent !== false) { if ($asReflectionObjects) { $classes[] = $parent; } else { $classes[] = $parent->getName(); } } else { $done = true; } } return $classes; } } 