<?php
 namespace PHPUnit\Framework; use Iterator; use IteratorAggregate; use PHPUnit\Runner\BaseTestRunner; use PHPUnit\Runner\Filter\Factory; use PHPUnit\Runner\PhptTestCase; use PHPUnit\Util\Fileloader; use PHPUnit\Util\InvalidArgumentHelper; use ReflectionClass; use ReflectionMethod; use Throwable; class TestSuite implements Test, SelfDescribing, IteratorAggregate { private $cachedNumTests; protected $backupGlobals; protected $backupStaticAttributes; private $beStrictAboutChangesToGlobalState; protected $runTestInSeparateProcess = false; protected $name = ''; protected $groups = []; protected $tests = []; protected $numTests = -1; protected $testCase = false; protected $foundClasses = []; private $iteratorFilter; private $declaredClasses; public function __construct($theClass = '', $name = '') { $this->declaredClasses = \get_declared_classes(); $argumentsValid = false; if (\is_object($theClass) && $theClass instanceof ReflectionClass) { $argumentsValid = true; } elseif (\is_string($theClass) && $theClass !== '' && \class_exists($theClass, false)) { $argumentsValid = true; if ($name == '') { $name = $theClass; } $theClass = new ReflectionClass($theClass); } elseif (\is_string($theClass)) { $this->setName($theClass); return; } if (!$argumentsValid) { throw new Exception; } if (!$theClass->isSubclassOf(TestCase::class)) { throw new Exception( 'Class "' . $theClass->name . '" does not extend PHPUnit\Framework\TestCase.' ); } if ($name != '') { $this->setName($name); } else { $this->setName($theClass->getName()); } $constructor = $theClass->getConstructor(); if ($constructor !== null && !$constructor->isPublic()) { $this->addTest( self::warning( \sprintf( 'Class "%s" has no public constructor.', $theClass->getName() ) ) ); return; } foreach ($theClass->getMethods() as $method) { $this->addTestMethod($theClass, $method); } if (empty($this->tests)) { $this->addTest( self::warning( \sprintf( 'No tests found in class "%s".', $theClass->getName() ) ) ); } $this->testCase = true; } public function toString() { return $this->getName(); } public function addTest(Test $test, $groups = []) { $class = new ReflectionClass($test); if (!$class->isAbstract()) { $this->tests[] = $test; $this->numTests = -1; if ($test instanceof self && empty($groups)) { $groups = $test->getGroups(); } if (empty($groups)) { $groups = ['default']; } foreach ($groups as $group) { if (!isset($this->groups[$group])) { $this->groups[$group] = [$test]; } else { $this->groups[$group][] = $test; } } if ($test instanceof TestCase) { $test->setGroups($groups); } } } public function addTestSuite($testClass) { if (\is_string($testClass) && \class_exists($testClass)) { $testClass = new ReflectionClass($testClass); } if (!\is_object($testClass)) { throw InvalidArgumentHelper::factory( 1, 'class name or object' ); } if ($testClass instanceof self) { $this->addTest($testClass); } elseif ($testClass instanceof ReflectionClass) { $suiteMethod = false; if (!$testClass->isAbstract() && $testClass->hasMethod(BaseTestRunner::SUITE_METHODNAME)) { $method = $testClass->getMethod( BaseTestRunner::SUITE_METHODNAME ); if ($method->isStatic()) { $this->addTest( $method->invoke(null, $testClass->getName()) ); $suiteMethod = true; } } if (!$suiteMethod && !$testClass->isAbstract() && $testClass->isSubclassOf(TestCase::class)) { $this->addTest(new self($testClass)); } } else { throw new Exception; } } public function addTestFile($filename) { if (!\is_string($filename)) { throw InvalidArgumentHelper::factory(1, 'string'); } if (\file_exists($filename) && \substr($filename, -5) == '.phpt') { $this->addTest( new PhptTestCase($filename) ); return; } $filename = Fileloader::checkAndLoad($filename); $newClasses = \array_diff(\get_declared_classes(), $this->declaredClasses); if (!empty($newClasses)) { $this->foundClasses = \array_merge($newClasses, $this->foundClasses); $this->declaredClasses = \get_declared_classes(); } $shortname = \basename($filename, '.php'); $shortnameRegEx = '/(?:^|_|\\\\)' . \preg_quote($shortname, '/') . '$/'; foreach ($this->foundClasses as $i => $className) { if (\preg_match($shortnameRegEx, $className)) { $class = new ReflectionClass($className); if ($class->getFileName() == $filename) { $newClasses = [$className]; unset($this->foundClasses[$i]); break; } } } foreach ($newClasses as $className) { $class = new ReflectionClass($className); if (\dirname($class->getFileName()) === __DIR__) { continue; } if (!$class->isAbstract()) { if ($class->hasMethod(BaseTestRunner::SUITE_METHODNAME)) { $method = $class->getMethod( BaseTestRunner::SUITE_METHODNAME ); if ($method->isStatic()) { $this->addTest($method->invoke(null, $className)); } } elseif ($class->implementsInterface(Test::class)) { $this->addTestSuite($class); } } } $this->numTests = -1; } public function addTestFiles($filenames) { if (!(\is_array($filenames) || (\is_object($filenames) && $filenames instanceof Iterator))) { throw InvalidArgumentHelper::factory( 1, 'array or iterator' ); } foreach ($filenames as $filename) { $this->addTestFile((string) $filename); } } public function count($preferCache = false) { if ($preferCache && $this->cachedNumTests !== null) { return $this->cachedNumTests; } $numTests = 0; foreach ($this as $test) { $numTests += \count($test); } $this->cachedNumTests = $numTests; return $numTests; } public static function createTest(ReflectionClass $theClass, $name) { $className = $theClass->getName(); if (!$theClass->isInstantiable()) { return self::warning( \sprintf('Cannot instantiate class "%s".', $className) ); } $backupSettings = \PHPUnit\Util\Test::getBackupSettings( $className, $name ); $preserveGlobalState = \PHPUnit\Util\Test::getPreserveGlobalStateSettings( $className, $name ); $runTestInSeparateProcess = \PHPUnit\Util\Test::getProcessIsolationSettings( $className, $name ); $runClassInSeparateProcess = \PHPUnit\Util\Test::getClassProcessIsolationSettings( $className, $name ); $constructor = $theClass->getConstructor(); if ($constructor !== null) { $parameters = $constructor->getParameters(); if (\count($parameters) < 2) { $test = new $className; } else { try { $data = \PHPUnit\Util\Test::getProvidedData( $className, $name ); } catch (IncompleteTestError $e) { $message = \sprintf( 'Test for %s::%s marked incomplete by data provider', $className, $name ); $_message = $e->getMessage(); if (!empty($_message)) { $message .= "\n" . $_message; } $data = self::incompleteTest($className, $name, $message); } catch (SkippedTestError $e) { $message = \sprintf( 'Test for %s::%s skipped by data provider', $className, $name ); $_message = $e->getMessage(); if (!empty($_message)) { $message .= "\n" . $_message; } $data = self::skipTest($className, $name, $message); } catch (Throwable $_t) { $t = $_t; } catch (Exception $_t) { $t = $_t; } if (isset($t)) { $message = \sprintf( 'The data provider specified for %s::%s is invalid.', $className, $name ); $_message = $t->getMessage(); if (!empty($_message)) { $message .= "\n" . $_message; } $data = self::warning($message); } if (isset($data)) { $test = new DataProviderTestSuite( $className . '::' . $name ); if (empty($data)) { $data = self::warning( \sprintf( 'No tests found in suite "%s".', $test->getName() ) ); } $groups = \PHPUnit\Util\Test::getGroups($className, $name); if ($data instanceof WarningTestCase || $data instanceof SkippedTestCase || $data instanceof IncompleteTestCase) { $test->addTest($data, $groups); } else { foreach ($data as $_dataName => $_data) { $_test = new $className($name, $_data, $_dataName); if ($runTestInSeparateProcess) { $_test->setRunTestInSeparateProcess(true); if ($preserveGlobalState !== null) { $_test->setPreserveGlobalState($preserveGlobalState); } } if ($runClassInSeparateProcess) { $_test->setRunClassInSeparateProcess(true); if ($preserveGlobalState !== null) { $_test->setPreserveGlobalState($preserveGlobalState); } } if ($backupSettings['backupGlobals'] !== null) { $_test->setBackupGlobals( $backupSettings['backupGlobals'] ); } if ($backupSettings['backupStaticAttributes'] !== null) { $_test->setBackupStaticAttributes( $backupSettings['backupStaticAttributes'] ); } $test->addTest($_test, $groups); } } } else { $test = new $className; } } } if (!isset($test)) { throw new Exception('No valid test provided.'); } if ($test instanceof TestCase) { $test->setName($name); if ($runTestInSeparateProcess) { $test->setRunTestInSeparateProcess(true); if ($preserveGlobalState !== null) { $test->setPreserveGlobalState($preserveGlobalState); } } if ($backupSettings['backupGlobals'] !== null) { $test->setBackupGlobals($backupSettings['backupGlobals']); } if ($backupSettings['backupStaticAttributes'] !== null) { $test->setBackupStaticAttributes( $backupSettings['backupStaticAttributes'] ); } } return $test; } protected function createResult() { return new TestResult; } public function getName() { return $this->name; } public function getGroups() { return \array_keys($this->groups); } public function getGroupDetails() { return $this->groups; } public function setGroupDetails(array $groups) { $this->groups = $groups; } public function run(TestResult $result = null) { if ($result === null) { $result = $this->createResult(); } if (\count($this) == 0) { return $result; } $hookMethods = \PHPUnit\Util\Test::getHookMethods($this->name); $result->startTestSuite($this); try { $this->setUp(); foreach ($hookMethods['beforeClass'] as $beforeClassMethod) { if ($this->testCase === true && \class_exists($this->name, false) && \method_exists($this->name, $beforeClassMethod)) { if ($missingRequirements = \PHPUnit\Util\Test::getMissingRequirements($this->name, $beforeClassMethod)) { $this->markTestSuiteSkipped(\implode(PHP_EOL, $missingRequirements)); } \call_user_func([$this->name, $beforeClassMethod]); } } } catch (SkippedTestSuiteError $e) { $numTests = \count($this); for ($i = 0; $i < $numTests; $i++) { $result->startTest($this); $result->addFailure($this, $e, 0); $result->endTest($this, 0); } $this->tearDown(); $result->endTestSuite($this); return $result; } catch (Throwable $_t) { $t = $_t; } catch (Exception $_t) { $t = $_t; } if (isset($t)) { $numTests = \count($this); for ($i = 0; $i < $numTests; $i++) { if ($result->shouldStop()) { break; } $result->startTest($this); $result->addError($this, $t, 0); $result->endTest($this, 0); } $this->tearDown(); $result->endTestSuite($this); return $result; } foreach ($this as $test) { if ($result->shouldStop()) { break; } if ($test instanceof TestCase || $test instanceof self) { $test->setBeStrictAboutChangesToGlobalState($this->beStrictAboutChangesToGlobalState); $test->setBackupGlobals($this->backupGlobals); $test->setBackupStaticAttributes($this->backupStaticAttributes); $test->setRunTestInSeparateProcess($this->runTestInSeparateProcess); } $test->run($result); } foreach ($hookMethods['afterClass'] as $afterClassMethod) { if ($this->testCase === true && \class_exists($this->name, false) && \method_exists($this->name, $afterClassMethod)) { \call_user_func([$this->name, $afterClassMethod]); } } $this->tearDown(); $result->endTestSuite($this); return $result; } public function setRunTestInSeparateProcess($runTestInSeparateProcess) { if (\is_bool($runTestInSeparateProcess)) { $this->runTestInSeparateProcess = $runTestInSeparateProcess; } else { throw InvalidArgumentHelper::factory(1, 'boolean'); } } public function runTest(Test $test, TestResult $result) { $test->run($result); } public function setName($name) { $this->name = $name; } public function testAt($index) { if (isset($this->tests[$index])) { return $this->tests[$index]; } return false; } public function tests() { return $this->tests; } public function setTests(array $tests) { $this->tests = $tests; } public function markTestSuiteSkipped($message = '') { throw new SkippedTestSuiteError($message); } protected function addTestMethod(ReflectionClass $class, ReflectionMethod $method) { if (!$this->isTestMethod($method)) { return; } $name = $method->getName(); if (!$method->isPublic()) { $this->addTest( self::warning( \sprintf( 'Test method "%s" in test class "%s" is not public.', $name, $class->getName() ) ) ); return; } $test = self::createTest($class, $name); if ($test instanceof TestCase || $test instanceof DataProviderTestSuite) { $test->setDependencies( \PHPUnit\Util\Test::getDependencies($class->getName(), $name) ); } $this->addTest( $test, \PHPUnit\Util\Test::getGroups($class->getName(), $name) ); } public static function isTestMethod(ReflectionMethod $method) { if (\strpos($method->name, 'test') === 0) { return true; } $docComment = $method->getDocComment(); return \strpos($docComment, '@test') !== false || \strpos($docComment, '@scenario') !== false; } protected static function warning($message) { return new WarningTestCase($message); } protected static function skipTest($class, $methodName, $message) { return new SkippedTestCase($class, $methodName, $message); } protected static function incompleteTest($class, $methodName, $message) { return new IncompleteTestCase($class, $methodName, $message); } public function setBeStrictAboutChangesToGlobalState($beStrictAboutChangesToGlobalState) { if (null === $this->beStrictAboutChangesToGlobalState && \is_bool($beStrictAboutChangesToGlobalState)) { $this->beStrictAboutChangesToGlobalState = $beStrictAboutChangesToGlobalState; } } public function setBackupGlobals($backupGlobals) { if (null === $this->backupGlobals && \is_bool($backupGlobals)) { $this->backupGlobals = $backupGlobals; } } public function setBackupStaticAttributes($backupStaticAttributes) { if (null === $this->backupStaticAttributes && \is_bool($backupStaticAttributes)) { $this->backupStaticAttributes = $backupStaticAttributes; } } public function getIterator() { $iterator = new TestSuiteIterator($this); if ($this->iteratorFilter !== null) { $iterator = $this->iteratorFilter->factory($iterator, $this); } return $iterator; } public function injectFilter(Factory $filter) { $this->iteratorFilter = $filter; foreach ($this as $test) { if ($test instanceof self) { $test->injectFilter($filter); } } } protected function setUp() { } protected function tearDown() { } } 