<?php
 namespace Doctrine\DBAL\Driver\OCI8; use Doctrine\DBAL\Driver\Statement; use Doctrine\DBAL\Driver\StatementIterator; use Doctrine\DBAL\FetchMode; use Doctrine\DBAL\ParameterType; use IteratorAggregate; use const OCI_ASSOC; use const OCI_B_BIN; use const OCI_B_BLOB; use const OCI_BOTH; use const OCI_D_LOB; use const OCI_FETCHSTATEMENT_BY_COLUMN; use const OCI_FETCHSTATEMENT_BY_ROW; use const OCI_NUM; use const OCI_RETURN_LOBS; use const OCI_RETURN_NULLS; use const OCI_TEMP_BLOB; use const PREG_OFFSET_CAPTURE; use const SQLT_CHR; use function array_key_exists; use function count; use function implode; use function is_numeric; use function oci_bind_by_name; use function oci_cancel; use function oci_error; use function oci_execute; use function oci_fetch_all; use function oci_fetch_array; use function oci_fetch_object; use function oci_new_descriptor; use function oci_num_fields; use function oci_num_rows; use function oci_parse; use function preg_match; use function preg_quote; use function sprintf; use function substr; class OCI8Statement implements IteratorAggregate, Statement { protected $_dbh; protected $_sth; protected $_conn; protected static $_PARAM = ':param'; protected static $fetchModeMap = [ FetchMode::MIXED => OCI_BOTH, FetchMode::ASSOCIATIVE => OCI_ASSOC, FetchMode::NUMERIC => OCI_NUM, FetchMode::COLUMN => OCI_NUM, ]; protected $_defaultFetchMode = FetchMode::MIXED; protected $_paramMap = []; private $boundValues = []; private $result = false; public function __construct($dbh, $statement, OCI8Connection $conn) { list($statement, $paramMap) = self::convertPositionalToNamedPlaceholders($statement); $this->_sth = oci_parse($dbh, $statement); $this->_dbh = $dbh; $this->_paramMap = $paramMap; $this->_conn = $conn; } public static function convertPositionalToNamedPlaceholders($statement) { $fragmentOffset = $tokenOffset = 0; $fragments = $paramMap = []; $currentLiteralDelimiter = null; do { if (!$currentLiteralDelimiter) { $result = self::findPlaceholderOrOpeningQuote( $statement, $tokenOffset, $fragmentOffset, $fragments, $currentLiteralDelimiter, $paramMap ); } else { $result = self::findClosingQuote($statement, $tokenOffset, $currentLiteralDelimiter); } } while ($result); if ($currentLiteralDelimiter) { throw new OCI8Exception(sprintf( 'The statement contains non-terminated string literal starting at offset %d', $tokenOffset - 1 )); } $fragments[] = substr($statement, $fragmentOffset); $statement = implode('', $fragments); return [$statement, $paramMap]; } private static function findPlaceholderOrOpeningQuote( $statement, &$tokenOffset, &$fragmentOffset, &$fragments, &$currentLiteralDelimiter, &$paramMap ) { $token = self::findToken($statement, $tokenOffset, '/[?\'"]/'); if (!$token) { return false; } if ($token === '?') { $position = count($paramMap) + 1; $param = ':param' . $position; $fragments[] = substr($statement, $fragmentOffset, $tokenOffset - $fragmentOffset); $fragments[] = $param; $paramMap[$position] = $param; $tokenOffset += 1; $fragmentOffset = $tokenOffset; return true; } $currentLiteralDelimiter = $token; ++$tokenOffset; return true; } private static function findClosingQuote( $statement, &$tokenOffset, &$currentLiteralDelimiter ) { $token = self::findToken( $statement, $tokenOffset, '/' . preg_quote($currentLiteralDelimiter, '/') . '/' ); if (!$token) { return false; } $currentLiteralDelimiter = false; ++$tokenOffset; return true; } private static function findToken($statement, &$offset, $regex) { if (preg_match($regex, $statement, $matches, PREG_OFFSET_CAPTURE, $offset)) { $offset = $matches[0][1]; return $matches[0][0]; } return null; } public function bindValue($param, $value, $type = ParameterType::STRING) { return $this->bindParam($param, $value, $type, null); } public function bindParam($column, &$variable, $type = ParameterType::STRING, $length = null) { $column = $this->_paramMap[$column] ?? $column; if ($type === ParameterType::LARGE_OBJECT) { $lob = oci_new_descriptor($this->_dbh, OCI_D_LOB); $lob->writeTemporary($variable, OCI_TEMP_BLOB); $variable =& $lob; } $this->boundValues[$column] =& $variable; return oci_bind_by_name( $this->_sth, $column, $variable, $length ?? -1, $this->convertParameterType($type) ); } private function convertParameterType(int $type) : int { switch ($type) { case ParameterType::BINARY: return OCI_B_BIN; case ParameterType::LARGE_OBJECT: return OCI_B_BLOB; default: return SQLT_CHR; } } public function closeCursor() { if (!$this->result) { return true; } oci_cancel($this->_sth); $this->result = false; return true; } public function columnCount() { return oci_num_fields($this->_sth); } public function errorCode() { $error = oci_error($this->_sth); if ($error !== false) { $error = $error['code']; } return $error; } public function errorInfo() { return oci_error($this->_sth); } public function execute($params = null) { if ($params) { $hasZeroIndex = array_key_exists(0, $params); foreach ($params as $key => $val) { if ($hasZeroIndex && is_numeric($key)) { $this->bindValue($key + 1, $val); } else { $this->bindValue($key, $val); } } } $ret = @oci_execute($this->_sth, $this->_conn->getExecuteMode()); if ( ! $ret) { throw OCI8Exception::fromErrorInfo($this->errorInfo()); } $this->result = true; return $ret; } public function setFetchMode($fetchMode, $arg2 = null, $arg3 = null) { $this->_defaultFetchMode = $fetchMode; return true; } public function getIterator() { return new StatementIterator($this); } public function fetch($fetchMode = null, $cursorOrientation = \PDO::FETCH_ORI_NEXT, $cursorOffset = 0) { if (!$this->result) { return false; } $fetchMode = $fetchMode ?: $this->_defaultFetchMode; if ($fetchMode === FetchMode::COLUMN) { return $this->fetchColumn(); } if ($fetchMode === FetchMode::STANDARD_OBJECT) { return oci_fetch_object($this->_sth); } if (! isset(self::$fetchModeMap[$fetchMode])) { throw new \InvalidArgumentException("Invalid fetch style: " . $fetchMode); } return oci_fetch_array( $this->_sth, self::$fetchModeMap[$fetchMode] | OCI_RETURN_NULLS | OCI_RETURN_LOBS ); } public function fetchAll($fetchMode = null, $fetchArgument = null, $ctorArgs = null) { $fetchMode = $fetchMode ?: $this->_defaultFetchMode; $result = []; if ($fetchMode === FetchMode::STANDARD_OBJECT) { while ($row = $this->fetch($fetchMode)) { $result[] = $row; } return $result; } if ( ! isset(self::$fetchModeMap[$fetchMode])) { throw new \InvalidArgumentException("Invalid fetch style: " . $fetchMode); } if (self::$fetchModeMap[$fetchMode] === OCI_BOTH) { while ($row = $this->fetch($fetchMode)) { $result[] = $row; } } else { $fetchStructure = OCI_FETCHSTATEMENT_BY_ROW; if ($fetchMode === FetchMode::COLUMN) { $fetchStructure = OCI_FETCHSTATEMENT_BY_COLUMN; } if (!$this->result) { return []; } oci_fetch_all($this->_sth, $result, 0, -1, self::$fetchModeMap[$fetchMode] | OCI_RETURN_NULLS | $fetchStructure | OCI_RETURN_LOBS); if ($fetchMode === FetchMode::COLUMN) { $result = $result[0]; } } return $result; } public function fetchColumn($columnIndex = 0) { if (!$this->result) { return false; } $row = oci_fetch_array($this->_sth, OCI_NUM | OCI_RETURN_NULLS | OCI_RETURN_LOBS); if (false === $row) { return false; } return $row[$columnIndex] ?? null; } public function rowCount() { return oci_num_rows($this->_sth); } } 