<?php
 namespace Illuminate\Encryption; use RuntimeException; use Illuminate\Contracts\Encryption\DecryptException; use Illuminate\Contracts\Encryption\EncryptException; use Illuminate\Contracts\Encryption\Encrypter as EncrypterContract; class Encrypter implements EncrypterContract { protected $key; protected $cipher; public function __construct($key, $cipher = 'AES-128-CBC') { $key = (string) $key; if (static::supported($key, $cipher)) { $this->key = $key; $this->cipher = $cipher; } else { throw new RuntimeException('The only supported ciphers are AES-128-CBC and AES-256-CBC with the correct key lengths.'); } } public static function supported($key, $cipher) { $length = mb_strlen($key, '8bit'); return ($cipher === 'AES-128-CBC' && $length === 16) || ($cipher === 'AES-256-CBC' && $length === 32); } public static function generateKey($cipher) { return random_bytes($cipher == 'AES-128-CBC' ? 16 : 32); } public function encrypt($value, $serialize = true) { $iv = random_bytes(openssl_cipher_iv_length($this->cipher)); $value = \openssl_encrypt( $serialize ? serialize($value) : $value, $this->cipher, $this->key, 0, $iv ); if ($value === false) { throw new EncryptException('Could not encrypt the data.'); } $mac = $this->hash($iv = base64_encode($iv), $value); $json = json_encode(compact('iv', 'value', 'mac')); if (json_last_error() !== JSON_ERROR_NONE) { throw new EncryptException('Could not encrypt the data.'); } return base64_encode($json); } public function encryptString($value) { return $this->encrypt($value, false); } public function decrypt($payload, $unserialize = true) { $payload = $this->getJsonPayload($payload); $iv = base64_decode($payload['iv']); $decrypted = \openssl_decrypt( $payload['value'], $this->cipher, $this->key, 0, $iv ); if ($decrypted === false) { throw new DecryptException('Could not decrypt the data.'); } return $unserialize ? unserialize($decrypted) : $decrypted; } public function decryptString($payload) { return $this->decrypt($payload, false); } protected function hash($iv, $value) { return hash_hmac('sha256', $iv.$value, $this->key); } protected function getJsonPayload($payload) { $payload = json_decode(base64_decode($payload), true); if (! $this->validPayload($payload)) { throw new DecryptException('The payload is invalid.'); } if (! $this->validMac($payload)) { throw new DecryptException('The MAC is invalid.'); } return $payload; } protected function validPayload($payload) { return is_array($payload) && isset($payload['iv'], $payload['value'], $payload['mac']) && strlen(base64_decode($payload['iv'], true)) === openssl_cipher_iv_length($this->cipher); } protected function validMac(array $payload) { $calculated = $this->calculateMac($payload, $bytes = random_bytes(16)); return hash_equals( hash_hmac('sha256', $payload['mac'], $bytes, true), $calculated ); } protected function calculateMac($payload, $bytes) { return hash_hmac( 'sha256', $this->hash($payload['iv'], $payload['value']), $bytes, true ); } public function getKey() { return $this->key; } } 