<?php
 namespace Illuminate\Foundation\Console; use Closure; use Illuminate\Support\Arr; use Illuminate\Support\Str; use Illuminate\Routing\Route; use Illuminate\Routing\Router; use Illuminate\Console\Command; use Symfony\Component\Console\Input\InputOption; class RouteListCommand extends Command { protected $name = 'route:list'; protected $description = 'List all registered routes'; protected $router; protected $routes; protected $headers = ['Domain', 'Method', 'URI', 'Name', 'Action', 'Middleware']; public function __construct(Router $router) { parent::__construct(); $this->router = $router; $this->routes = $router->getRoutes(); } public function handle() { if (count($this->routes) == 0) { return $this->error("Your application doesn't have any routes."); } $this->displayRoutes($this->getRoutes()); } protected function getRoutes() { $routes = collect($this->routes)->map(function ($route) { return $this->getRouteInformation($route); })->all(); if ($sort = $this->option('sort')) { $routes = $this->sortRoutes($sort, $routes); } if ($this->option('reverse')) { $routes = array_reverse($routes); } return array_filter($routes); } protected function getRouteInformation(Route $route) { return $this->filterRoute([ 'host' => $route->domain(), 'method' => implode('|', $route->methods()), 'uri' => $route->uri(), 'name' => $route->getName(), 'action' => $route->getActionName(), 'middleware' => $this->getMiddleware($route), ]); } protected function sortRoutes($sort, $routes) { return Arr::sort($routes, function ($route) use ($sort) { return $route[$sort]; }); } protected function displayRoutes(array $routes) { $this->table($this->headers, $routes); } protected function getMiddleware($route) { return collect($route->gatherMiddleware())->map(function ($middleware) { return $middleware instanceof Closure ? 'Closure' : $middleware; })->implode(','); } protected function filterRoute(array $route) { if (($this->option('name') && ! Str::contains($route['name'], $this->option('name'))) || $this->option('path') && ! Str::contains($route['uri'], $this->option('path')) || $this->option('method') && ! Str::contains($route['method'], strtoupper($this->option('method')))) { return; } return $route; } protected function getOptions() { return [ ['method', null, InputOption::VALUE_OPTIONAL, 'Filter the routes by method.'], ['name', null, InputOption::VALUE_OPTIONAL, 'Filter the routes by name.'], ['path', null, InputOption::VALUE_OPTIONAL, 'Filter the routes by path.'], ['reverse', 'r', InputOption::VALUE_NONE, 'Reverse the ordering of the routes.'], ['sort', null, InputOption::VALUE_OPTIONAL, 'The column (host, method, uri, name, action, middleware) to sort by.', 'uri'], ]; } } 