<?php
 namespace Symfony\Component\Filesystem\Tests; class FilesystemTest extends FilesystemTestCase { public function testCopyCreatesNewFile() { $sourceFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_source_file'; $targetFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_target_file'; file_put_contents($sourceFilePath, 'SOURCE FILE'); $this->filesystem->copy($sourceFilePath, $targetFilePath); $this->assertFileExists($targetFilePath); $this->assertStringEqualsFile($targetFilePath, 'SOURCE FILE'); } public function testCopyFails() { $sourceFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_source_file'; $targetFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_target_file'; $this->filesystem->copy($sourceFilePath, $targetFilePath); } public function testCopyUnreadableFileFails() { if ('\\' === \DIRECTORY_SEPARATOR) { $this->markTestSkipped('This test cannot run on Windows.'); } if (!getenv('USER') || 'root' === getenv('USER')) { $this->markTestSkipped('This test will fail if run under superuser'); } $sourceFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_source_file'; $targetFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_target_file'; file_put_contents($sourceFilePath, 'SOURCE FILE'); $this->filesystem->chmod($sourceFilePath, 0222); $this->filesystem->copy($sourceFilePath, $targetFilePath); } public function testCopyOverridesExistingFileIfModified() { $sourceFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_source_file'; $targetFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_target_file'; file_put_contents($sourceFilePath, 'SOURCE FILE'); file_put_contents($targetFilePath, 'TARGET FILE'); touch($targetFilePath, time() - 1000); $this->filesystem->copy($sourceFilePath, $targetFilePath); $this->assertFileExists($targetFilePath); $this->assertStringEqualsFile($targetFilePath, 'SOURCE FILE'); } public function testCopyDoesNotOverrideExistingFileByDefault() { $sourceFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_source_file'; $targetFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_target_file'; file_put_contents($sourceFilePath, 'SOURCE FILE'); file_put_contents($targetFilePath, 'TARGET FILE'); $modificationTime = time() - 1000; touch($sourceFilePath, $modificationTime); touch($targetFilePath, $modificationTime); $this->filesystem->copy($sourceFilePath, $targetFilePath); $this->assertFileExists($targetFilePath); $this->assertStringEqualsFile($targetFilePath, 'TARGET FILE'); } public function testCopyOverridesExistingFileIfForced() { $sourceFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_source_file'; $targetFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_target_file'; file_put_contents($sourceFilePath, 'SOURCE FILE'); file_put_contents($targetFilePath, 'TARGET FILE'); $modificationTime = time() - 1000; touch($sourceFilePath, $modificationTime); touch($targetFilePath, $modificationTime); $this->filesystem->copy($sourceFilePath, $targetFilePath, true); $this->assertFileExists($targetFilePath); $this->assertStringEqualsFile($targetFilePath, 'SOURCE FILE'); } public function testCopyWithOverrideWithReadOnlyTargetFails() { if ('\\' === \DIRECTORY_SEPARATOR) { $this->markTestSkipped('This test cannot run on Windows.'); } if (!getenv('USER') || 'root' === getenv('USER')) { $this->markTestSkipped('This test will fail if run under superuser'); } $sourceFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_source_file'; $targetFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_target_file'; file_put_contents($sourceFilePath, 'SOURCE FILE'); file_put_contents($targetFilePath, 'TARGET FILE'); $modificationTime = time() - 1000; touch($sourceFilePath, $modificationTime); touch($targetFilePath, $modificationTime); $this->filesystem->chmod($targetFilePath, 0444); $this->filesystem->copy($sourceFilePath, $targetFilePath, true); } public function testCopyCreatesTargetDirectoryIfItDoesNotExist() { $sourceFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_source_file'; $targetFileDirectory = $this->workspace.\DIRECTORY_SEPARATOR.'directory'; $targetFilePath = $targetFileDirectory.\DIRECTORY_SEPARATOR.'copy_target_file'; file_put_contents($sourceFilePath, 'SOURCE FILE'); $this->filesystem->copy($sourceFilePath, $targetFilePath); $this->assertTrue(is_dir($targetFileDirectory)); $this->assertFileExists($targetFilePath); $this->assertStringEqualsFile($targetFilePath, 'SOURCE FILE'); } public function testCopyForOriginUrlsAndExistingLocalFileDefaultsToCopy() { if (!\in_array('https', stream_get_wrappers())) { $this->markTestSkipped('"https" stream wrapper is not enabled.'); } $sourceFilePath = 'https://symfony.com/images/common/logo/logo_symfony_header.png'; $targetFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_target_file'; file_put_contents($targetFilePath, 'TARGET FILE'); $this->filesystem->copy($sourceFilePath, $targetFilePath, false); $this->assertFileExists($targetFilePath); $this->assertEquals(file_get_contents($sourceFilePath), file_get_contents($targetFilePath)); } public function testMkdirCreatesDirectoriesRecursively() { $directory = $this->workspace .\DIRECTORY_SEPARATOR.'directory' .\DIRECTORY_SEPARATOR.'sub_directory'; $this->filesystem->mkdir($directory); $this->assertTrue(is_dir($directory)); } public function testMkdirCreatesDirectoriesFromArray() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR; $directories = array( $basePath.'1', $basePath.'2', $basePath.'3', ); $this->filesystem->mkdir($directories); $this->assertTrue(is_dir($basePath.'1')); $this->assertTrue(is_dir($basePath.'2')); $this->assertTrue(is_dir($basePath.'3')); } public function testMkdirCreatesDirectoriesFromTraversableObject() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR; $directories = new \ArrayObject(array( $basePath.'1', $basePath.'2', $basePath.'3', )); $this->filesystem->mkdir($directories); $this->assertTrue(is_dir($basePath.'1')); $this->assertTrue(is_dir($basePath.'2')); $this->assertTrue(is_dir($basePath.'3')); } public function testMkdirCreatesDirectoriesFails() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR; $dir = $basePath.'2'; file_put_contents($dir, ''); $this->filesystem->mkdir($dir); } public function testTouchCreatesEmptyFile() { $file = $this->workspace.\DIRECTORY_SEPARATOR.'1'; $this->filesystem->touch($file); $this->assertFileExists($file); } public function testTouchFails() { $file = $this->workspace.\DIRECTORY_SEPARATOR.'1'.\DIRECTORY_SEPARATOR.'2'; $this->filesystem->touch($file); } public function testTouchCreatesEmptyFilesFromArray() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR; $files = array( $basePath.'1', $basePath.'2', $basePath.'3', ); $this->filesystem->touch($files); $this->assertFileExists($basePath.'1'); $this->assertFileExists($basePath.'2'); $this->assertFileExists($basePath.'3'); } public function testTouchCreatesEmptyFilesFromTraversableObject() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR; $files = new \ArrayObject(array( $basePath.'1', $basePath.'2', $basePath.'3', )); $this->filesystem->touch($files); $this->assertFileExists($basePath.'1'); $this->assertFileExists($basePath.'2'); $this->assertFileExists($basePath.'3'); } public function testRemoveCleansFilesAndDirectoriesIteratively() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR.'directory'.\DIRECTORY_SEPARATOR; mkdir($basePath); mkdir($basePath.'dir'); touch($basePath.'file'); $this->filesystem->remove($basePath); $this->assertFileNotExists($basePath); } public function testRemoveCleansArrayOfFilesAndDirectories() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR; mkdir($basePath.'dir'); touch($basePath.'file'); $files = array( $basePath.'dir', $basePath.'file', ); $this->filesystem->remove($files); $this->assertFileNotExists($basePath.'dir'); $this->assertFileNotExists($basePath.'file'); } public function testRemoveCleansTraversableObjectOfFilesAndDirectories() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR; mkdir($basePath.'dir'); touch($basePath.'file'); $files = new \ArrayObject(array( $basePath.'dir', $basePath.'file', )); $this->filesystem->remove($files); $this->assertFileNotExists($basePath.'dir'); $this->assertFileNotExists($basePath.'file'); } public function testRemoveIgnoresNonExistingFiles() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR; mkdir($basePath.'dir'); $files = array( $basePath.'dir', $basePath.'file', ); $this->filesystem->remove($files); $this->assertFileNotExists($basePath.'dir'); } public function testRemoveCleansInvalidLinks() { $this->markAsSkippedIfSymlinkIsMissing(); $basePath = $this->workspace.\DIRECTORY_SEPARATOR.'directory'.\DIRECTORY_SEPARATOR; mkdir($basePath); mkdir($basePath.'dir'); @symlink($basePath.'file', $basePath.'file-link'); $this->filesystem->symlink($basePath.'dir/', $basePath.'dir-link'); $this->assertTrue(is_dir($basePath.'dir-link')); rmdir($basePath.'dir'); $this->assertFalse('\\' === \DIRECTORY_SEPARATOR ? @readlink($basePath.'dir-link') : is_dir($basePath.'dir-link')); $this->filesystem->remove($basePath); $this->assertFileNotExists($basePath); } public function testFilesExists() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR.'directory'.\DIRECTORY_SEPARATOR; mkdir($basePath); touch($basePath.'file1'); mkdir($basePath.'folder'); $this->assertTrue($this->filesystem->exists($basePath.'file1')); $this->assertTrue($this->filesystem->exists($basePath.'folder')); } public function testFilesExistsFails() { if ('\\' !== \DIRECTORY_SEPARATOR) { $this->markTestSkipped('Long file names are an issue on Windows'); } $basePath = $this->workspace.'\\directory\\'; $maxPathLength = PHP_MAXPATHLEN - 2; $oldPath = getcwd(); mkdir($basePath); chdir($basePath); $file = str_repeat('T', $maxPathLength - \strlen($basePath) + 1); $path = $basePath.$file; exec('TYPE NUL >>'.$file); $this->longPathNamesWindows[] = $path; chdir($oldPath); $this->filesystem->exists($path); } public function testFilesExistsTraversableObjectOfFilesAndDirectories() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR; mkdir($basePath.'dir'); touch($basePath.'file'); $files = new \ArrayObject(array( $basePath.'dir', $basePath.'file', )); $this->assertTrue($this->filesystem->exists($files)); } public function testFilesNotExistsTraversableObjectOfFilesAndDirectories() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR; mkdir($basePath.'dir'); touch($basePath.'file'); touch($basePath.'file2'); $files = new \ArrayObject(array( $basePath.'dir', $basePath.'file', $basePath.'file2', )); unlink($basePath.'file'); $this->assertFalse($this->filesystem->exists($files)); } public function testInvalidFileNotExists() { $basePath = $this->workspace.\DIRECTORY_SEPARATOR.'directory'.\DIRECTORY_SEPARATOR; $this->assertFalse($this->filesystem->exists($basePath.time())); } public function testChmodChangesFileMode() { $this->markAsSkippedIfChmodIsMissing(); $dir = $this->workspace.\DIRECTORY_SEPARATOR.'dir'; mkdir($dir); $file = $dir.\DIRECTORY_SEPARATOR.'file'; touch($file); $this->filesystem->chmod($file, 0400); $this->filesystem->chmod($dir, 0753); $this->assertFilePermissions(753, $dir); $this->assertFilePermissions(400, $file); } public function testChmodWithWrongModLeavesPreviousPermissionsUntouched() { $this->markAsSkippedIfChmodIsMissing(); $dir = $this->workspace.\DIRECTORY_SEPARATOR.'file'; touch($dir); $permissions = fileperms($dir); $this->filesystem->chmod($dir, 'Wrongmode'); $this->assertSame($permissions, fileperms($dir)); } public function testChmodRecursive() { $this->markAsSkippedIfChmodIsMissing(); $dir = $this->workspace.\DIRECTORY_SEPARATOR.'dir'; mkdir($dir); $file = $dir.\DIRECTORY_SEPARATOR.'file'; touch($file); $this->filesystem->chmod($file, 0400, 0000, true); $this->filesystem->chmod($dir, 0753, 0000, true); $this->assertFilePermissions(753, $dir); $this->assertFilePermissions(753, $file); } public function testChmodAppliesUmask() { $this->markAsSkippedIfChmodIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; touch($file); $this->filesystem->chmod($file, 0770, 0022); $this->assertFilePermissions(750, $file); } public function testChmodChangesModeOfArrayOfFiles() { $this->markAsSkippedIfChmodIsMissing(); $directory = $this->workspace.\DIRECTORY_SEPARATOR.'directory'; $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $files = array($directory, $file); mkdir($directory); touch($file); $this->filesystem->chmod($files, 0753); $this->assertFilePermissions(753, $file); $this->assertFilePermissions(753, $directory); } public function testChmodChangesModeOfTraversableFileObject() { $this->markAsSkippedIfChmodIsMissing(); $directory = $this->workspace.\DIRECTORY_SEPARATOR.'directory'; $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $files = new \ArrayObject(array($directory, $file)); mkdir($directory); touch($file); $this->filesystem->chmod($files, 0753); $this->assertFilePermissions(753, $file); $this->assertFilePermissions(753, $directory); } public function testChmodChangesZeroModeOnSubdirectoriesOnRecursive() { $this->markAsSkippedIfChmodIsMissing(); $directory = $this->workspace.\DIRECTORY_SEPARATOR.'directory'; $subdirectory = $directory.\DIRECTORY_SEPARATOR.'subdirectory'; mkdir($directory); mkdir($subdirectory); chmod($subdirectory, 0000); $this->filesystem->chmod($directory, 0753, 0000, true); $this->assertFilePermissions(753, $subdirectory); } public function testChown() { $this->markAsSkippedIfPosixIsMissing(); $dir = $this->workspace.\DIRECTORY_SEPARATOR.'dir'; mkdir($dir); $owner = $this->getFileOwner($dir); $this->filesystem->chown($dir, $owner); $this->assertSame($owner, $this->getFileOwner($dir)); } public function testChownRecursive() { $this->markAsSkippedIfPosixIsMissing(); $dir = $this->workspace.\DIRECTORY_SEPARATOR.'dir'; mkdir($dir); $file = $dir.\DIRECTORY_SEPARATOR.'file'; touch($file); $owner = $this->getFileOwner($dir); $this->filesystem->chown($dir, $owner, true); $this->assertSame($owner, $this->getFileOwner($file)); } public function testChownSymlink() { $this->markAsSkippedIfSymlinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); $this->filesystem->symlink($file, $link); $owner = $this->getFileOwner($link); $this->filesystem->chown($link, $owner); $this->assertSame($owner, $this->getFileOwner($link)); } public function testChownLink() { $this->markAsSkippedIfLinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); $this->filesystem->hardlink($file, $link); $owner = $this->getFileOwner($link); $this->filesystem->chown($link, $owner); $this->assertSame($owner, $this->getFileOwner($link)); } public function testChownSymlinkFails() { $this->markAsSkippedIfSymlinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); $this->filesystem->symlink($file, $link); $this->filesystem->chown($link, 'user'.time().mt_rand(1000, 9999)); } public function testChownLinkFails() { $this->markAsSkippedIfLinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); $this->filesystem->hardlink($file, $link); $this->filesystem->chown($link, 'user'.time().mt_rand(1000, 9999)); } public function testChownFail() { $this->markAsSkippedIfPosixIsMissing(); $dir = $this->workspace.\DIRECTORY_SEPARATOR.'dir'; mkdir($dir); $this->filesystem->chown($dir, 'user'.time().mt_rand(1000, 9999)); } public function testChgrp() { $this->markAsSkippedIfPosixIsMissing(); $dir = $this->workspace.\DIRECTORY_SEPARATOR.'dir'; mkdir($dir); $group = $this->getFileGroup($dir); $this->filesystem->chgrp($dir, $group); $this->assertSame($group, $this->getFileGroup($dir)); } public function testChgrpRecursive() { $this->markAsSkippedIfPosixIsMissing(); $dir = $this->workspace.\DIRECTORY_SEPARATOR.'dir'; mkdir($dir); $file = $dir.\DIRECTORY_SEPARATOR.'file'; touch($file); $group = $this->getFileGroup($dir); $this->filesystem->chgrp($dir, $group, true); $this->assertSame($group, $this->getFileGroup($file)); } public function testChgrpSymlink() { $this->markAsSkippedIfSymlinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); $this->filesystem->symlink($file, $link); $group = $this->getFileGroup($link); $this->filesystem->chgrp($link, $group); $this->assertSame($group, $this->getFileGroup($link)); } public function testChgrpLink() { $this->markAsSkippedIfLinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); $this->filesystem->hardlink($file, $link); $group = $this->getFileGroup($link); $this->filesystem->chgrp($link, $group); $this->assertSame($group, $this->getFileGroup($link)); } public function testChgrpSymlinkFails() { $this->markAsSkippedIfSymlinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); $this->filesystem->symlink($file, $link); $this->filesystem->chgrp($link, 'user'.time().mt_rand(1000, 9999)); } public function testChgrpLinkFails() { $this->markAsSkippedIfLinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); $this->filesystem->hardlink($file, $link); $this->filesystem->chgrp($link, 'user'.time().mt_rand(1000, 9999)); } public function testChgrpFail() { $this->markAsSkippedIfPosixIsMissing(); $dir = $this->workspace.\DIRECTORY_SEPARATOR.'dir'; mkdir($dir); $this->filesystem->chgrp($dir, 'user'.time().mt_rand(1000, 9999)); } public function testRename() { $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $newPath = $this->workspace.\DIRECTORY_SEPARATOR.'new_file'; touch($file); $this->filesystem->rename($file, $newPath); $this->assertFileNotExists($file); $this->assertFileExists($newPath); } public function testRenameThrowsExceptionIfTargetAlreadyExists() { $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $newPath = $this->workspace.\DIRECTORY_SEPARATOR.'new_file'; touch($file); touch($newPath); $this->filesystem->rename($file, $newPath); } public function testRenameOverwritesTheTargetIfItAlreadyExists() { $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $newPath = $this->workspace.\DIRECTORY_SEPARATOR.'new_file'; touch($file); touch($newPath); $this->filesystem->rename($file, $newPath, true); $this->assertFileNotExists($file); $this->assertFileExists($newPath); } public function testRenameThrowsExceptionOnError() { $file = $this->workspace.\DIRECTORY_SEPARATOR.uniqid('fs_test_', true); $newPath = $this->workspace.\DIRECTORY_SEPARATOR.'new_file'; $this->filesystem->rename($file, $newPath); } public function testSymlink() { if ('\\' === \DIRECTORY_SEPARATOR) { $this->markTestSkipped('Windows does not support creating "broken" symlinks'); } $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; $this->filesystem->symlink($file, $link); $this->assertTrue(is_link($link)); touch($file); $this->assertEquals($file, readlink($link)); } public function testRemoveSymlink() { $this->markAsSkippedIfSymlinkIsMissing(); $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; $this->filesystem->remove($link); $this->assertFalse(is_link($link)); $this->assertFalse(is_file($link)); $this->assertFalse(is_dir($link)); } public function testSymlinkIsOverwrittenIfPointsToDifferentTarget() { $this->markAsSkippedIfSymlinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); symlink($this->workspace, $link); $this->filesystem->symlink($file, $link); $this->assertTrue(is_link($link)); $this->assertEquals($file, readlink($link)); } public function testSymlinkIsNotOverwrittenIfAlreadyCreated() { $this->markAsSkippedIfSymlinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); symlink($file, $link); $this->filesystem->symlink($file, $link); $this->assertTrue(is_link($link)); $this->assertEquals($file, readlink($link)); } public function testSymlinkCreatesTargetDirectoryIfItDoesNotExist() { $this->markAsSkippedIfSymlinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link1 = $this->workspace.\DIRECTORY_SEPARATOR.'dir'.\DIRECTORY_SEPARATOR.'link'; $link2 = $this->workspace.\DIRECTORY_SEPARATOR.'dir'.\DIRECTORY_SEPARATOR.'subdir'.\DIRECTORY_SEPARATOR.'link'; touch($file); $this->filesystem->symlink($file, $link1); $this->filesystem->symlink($file, $link2); $this->assertTrue(is_link($link1)); $this->assertEquals($file, readlink($link1)); $this->assertTrue(is_link($link2)); $this->assertEquals($file, readlink($link2)); } public function testLink() { $this->markAsSkippedIfLinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); $this->filesystem->hardlink($file, $link); $this->assertTrue(is_file($link)); $this->assertEquals(fileinode($file), fileinode($link)); } public function testRemoveLink() { $this->markAsSkippedIfLinkIsMissing(); $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; $this->filesystem->remove($link); $this->assertTrue(!is_file($link)); } public function testLinkIsOverwrittenIfPointsToDifferentTarget() { $this->markAsSkippedIfLinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $file2 = $this->workspace.\DIRECTORY_SEPARATOR.'file2'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); touch($file2); link($file2, $link); $this->filesystem->hardlink($file, $link); $this->assertTrue(is_file($link)); $this->assertEquals(fileinode($file), fileinode($link)); } public function testLinkIsNotOverwrittenIfAlreadyCreated() { $this->markAsSkippedIfLinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); link($file, $link); $this->filesystem->hardlink($file, $link); $this->assertTrue(is_file($link)); $this->assertEquals(fileinode($file), fileinode($link)); } public function testLinkWithSeveralTargets() { $this->markAsSkippedIfLinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link1 = $this->workspace.\DIRECTORY_SEPARATOR.'link'; $link2 = $this->workspace.\DIRECTORY_SEPARATOR.'link2'; touch($file); $this->filesystem->hardlink($file, array($link1, $link2)); $this->assertTrue(is_file($link1)); $this->assertEquals(fileinode($file), fileinode($link1)); $this->assertTrue(is_file($link2)); $this->assertEquals(fileinode($file), fileinode($link2)); } public function testLinkWithSameTarget() { $this->markAsSkippedIfLinkIsMissing(); $file = $this->workspace.\DIRECTORY_SEPARATOR.'file'; $link = $this->workspace.\DIRECTORY_SEPARATOR.'link'; touch($file); $this->filesystem->hardlink($file, array($link, $link)); $this->assertTrue(is_file($link)); $this->assertEquals(fileinode($file), fileinode($link)); } public function testReadRelativeLink() { $this->markAsSkippedIfSymlinkIsMissing(); if ('\\' === \DIRECTORY_SEPARATOR) { $this->markTestSkipped('Relative symbolic links are not supported on Windows'); } $file = $this->workspace.'/file'; $link1 = $this->workspace.'/dir/link'; $link2 = $this->workspace.'/dir/link2'; touch($file); $this->filesystem->symlink('../file', $link1); $this->filesystem->symlink('link', $link2); $this->assertEquals($this->normalize('../file'), $this->filesystem->readlink($link1)); $this->assertEquals('link', $this->filesystem->readlink($link2)); $this->assertEquals($file, $this->filesystem->readlink($link1, true)); $this->assertEquals($file, $this->filesystem->readlink($link2, true)); $this->assertEquals($file, $this->filesystem->readlink($file, true)); } public function testReadAbsoluteLink() { $this->markAsSkippedIfSymlinkIsMissing(); $file = $this->normalize($this->workspace.'/file'); $link1 = $this->normalize($this->workspace.'/dir/link'); $link2 = $this->normalize($this->workspace.'/dir/link2'); touch($file); $this->filesystem->symlink($file, $link1); $this->filesystem->symlink($link1, $link2); $this->assertEquals($file, $this->filesystem->readlink($link1)); $this->assertEquals($link1, $this->filesystem->readlink($link2)); $this->assertEquals($file, $this->filesystem->readlink($link1, true)); $this->assertEquals($file, $this->filesystem->readlink($link2, true)); $this->assertEquals($file, $this->filesystem->readlink($file, true)); } public function testReadBrokenLink() { $this->markAsSkippedIfSymlinkIsMissing(); if ('\\' === \DIRECTORY_SEPARATOR) { $this->markTestSkipped('Windows does not support creating "broken" symlinks'); } $file = $this->workspace.'/file'; $link = $this->workspace.'/link'; $this->filesystem->symlink($file, $link); $this->assertEquals($file, $this->filesystem->readlink($link)); $this->assertNull($this->filesystem->readlink($link, true)); touch($file); $this->assertEquals($file, $this->filesystem->readlink($link, true)); } public function testReadLinkDefaultPathDoesNotExist() { $this->assertNull($this->filesystem->readlink($this->normalize($this->workspace.'/invalid'))); } public function testReadLinkDefaultPathNotLink() { $file = $this->normalize($this->workspace.'/file'); touch($file); $this->assertNull($this->filesystem->readlink($file)); } public function testReadLinkCanonicalizePath() { $this->markAsSkippedIfSymlinkIsMissing(); $file = $this->normalize($this->workspace.'/file'); mkdir($this->normalize($this->workspace.'/dir')); touch($file); $this->assertEquals($file, $this->filesystem->readlink($this->normalize($this->workspace.'/dir/../file'), true)); } public function testReadLinkCanonicalizedPathDoesNotExist() { $this->assertNull($this->filesystem->readlink($this->normalize($this->workspace.'invalid'), true)); } public function testMakePathRelative($endPath, $startPath, $expectedPath) { $path = $this->filesystem->makePathRelative($endPath, $startPath); $this->assertEquals($expectedPath, $path); } public function providePathsForMakePathRelative() { $paths = array( array('/var/lib/symfony/src/Symfony/', '/var/lib/symfony/src/Symfony/Component', '../'), array('/var/lib/symfony/src/Symfony/', '/var/lib/symfony/src/Symfony/Component/', '../'), array('/var/lib/symfony/src/Symfony', '/var/lib/symfony/src/Symfony/Component', '../'), array('/var/lib/symfony/src/Symfony', '/var/lib/symfony/src/Symfony/Component/', '../'), array('/usr/lib/symfony/', '/var/lib/symfony/src/Symfony/Component', '../../../../../../usr/lib/symfony/'), array('/var/lib/symfony/src/Symfony/', '/var/lib/symfony/', 'src/Symfony/'), array('/aa/bb', '/aa/bb', './'), array('/aa/bb', '/aa/bb/', './'), array('/aa/bb/', '/aa/bb', './'), array('/aa/bb/', '/aa/bb/', './'), array('/aa/bb/cc', '/aa/bb/cc/dd', '../'), array('/aa/bb/cc', '/aa/bb/cc/dd/', '../'), array('/aa/bb/cc/', '/aa/bb/cc/dd', '../'), array('/aa/bb/cc/', '/aa/bb/cc/dd/', '../'), array('/aa/bb/cc', '/aa', 'bb/cc/'), array('/aa/bb/cc', '/aa/', 'bb/cc/'), array('/aa/bb/cc/', '/aa', 'bb/cc/'), array('/aa/bb/cc/', '/aa/', 'bb/cc/'), array('/a/aab/bb', '/a/aa', '../aab/bb/'), array('/a/aab/bb', '/a/aa/', '../aab/bb/'), array('/a/aab/bb/', '/a/aa', '../aab/bb/'), array('/a/aab/bb/', '/a/aa/', '../aab/bb/'), array('/a/aab/bb/', '/', 'a/aab/bb/'), array('/a/aab/bb/', '/b/aab', '../../a/aab/bb/'), array('/aab/bb', '/aa', '../aab/bb/'), array('/aab', '/aa', '../aab/'), array('/aa/bb/cc', '/aa/dd/..', 'bb/cc/'), array('/aa/../bb/cc', '/aa/dd/..', '../bb/cc/'), array('/aa/bb/../../cc', '/aa/../dd/..', 'cc/'), array('/../aa/bb/cc', '/aa/dd/..', 'bb/cc/'), array('/../../aa/../bb/cc', '/aa/dd/..', '../bb/cc/'), array('C:/aa/bb/cc', 'C:/aa/dd/..', 'bb/cc/'), array('c:/aa/../bb/cc', 'c:/aa/dd/..', '../bb/cc/'), array('C:/aa/bb/../../cc', 'C:/aa/../dd/..', 'cc/'), array('C:/../aa/bb/cc', 'C:/aa/dd/..', 'bb/cc/'), array('C:/../../aa/../bb/cc', 'C:/aa/dd/..', '../bb/cc/'), ); if ('\\' === \DIRECTORY_SEPARATOR) { $paths[] = array('c:\var\lib/symfony/src/Symfony/', 'c:/var/lib/symfony/', 'src/Symfony/'); } return $paths; } public function testMakePathRelativeWithRelativeStartPath() { $this->assertSame('../../../', $this->filesystem->makePathRelative('/var/lib/symfony/', 'var/lib/symfony/src/Symfony/Component')); } public function testMakePathRelativeWithRelativeEndPath() { $this->assertSame('../../../', $this->filesystem->makePathRelative('var/lib/symfony/', '/var/lib/symfony/src/Symfony/Component')); } public function testMirrorCopiesFilesAndDirectoriesRecursively() { $sourcePath = $this->workspace.\DIRECTORY_SEPARATOR.'source'.\DIRECTORY_SEPARATOR; $directory = $sourcePath.'directory'.\DIRECTORY_SEPARATOR; $file1 = $directory.'file1'; $file2 = $sourcePath.'file2'; mkdir($sourcePath); mkdir($directory); file_put_contents($file1, 'FILE1'); file_put_contents($file2, 'FILE2'); $targetPath = $this->workspace.\DIRECTORY_SEPARATOR.'target'.\DIRECTORY_SEPARATOR; $this->filesystem->mirror($sourcePath, $targetPath); $this->assertTrue(is_dir($targetPath)); $this->assertTrue(is_dir($targetPath.'directory')); $this->assertFileEquals($file1, $targetPath.'directory'.\DIRECTORY_SEPARATOR.'file1'); $this->assertFileEquals($file2, $targetPath.'file2'); $this->filesystem->remove($file1); $this->filesystem->mirror($sourcePath, $targetPath, null, array('delete' => false)); $this->assertTrue($this->filesystem->exists($targetPath.'directory'.\DIRECTORY_SEPARATOR.'file1')); $this->filesystem->mirror($sourcePath, $targetPath, null, array('delete' => true)); $this->assertFalse($this->filesystem->exists($targetPath.'directory'.\DIRECTORY_SEPARATOR.'file1')); file_put_contents($file1, 'FILE1'); $this->filesystem->mirror($sourcePath, $targetPath, null, array('delete' => true)); $this->assertTrue($this->filesystem->exists($targetPath.'directory'.\DIRECTORY_SEPARATOR.'file1')); $this->filesystem->remove($directory); $this->filesystem->mirror($sourcePath, $targetPath, null, array('delete' => true)); $this->assertFalse($this->filesystem->exists($targetPath.'directory')); $this->assertFalse($this->filesystem->exists($targetPath.'directory'.\DIRECTORY_SEPARATOR.'file1')); } public function testMirrorCreatesEmptyDirectory() { $sourcePath = $this->workspace.\DIRECTORY_SEPARATOR.'source'.\DIRECTORY_SEPARATOR; mkdir($sourcePath); $targetPath = $this->workspace.\DIRECTORY_SEPARATOR.'target'.\DIRECTORY_SEPARATOR; $this->filesystem->mirror($sourcePath, $targetPath); $this->assertTrue(is_dir($targetPath)); $this->filesystem->remove($sourcePath); } public function testMirrorCopiesLinks() { $this->markAsSkippedIfSymlinkIsMissing(); $sourcePath = $this->workspace.\DIRECTORY_SEPARATOR.'source'.\DIRECTORY_SEPARATOR; mkdir($sourcePath); file_put_contents($sourcePath.'file1', 'FILE1'); symlink($sourcePath.'file1', $sourcePath.'link1'); $targetPath = $this->workspace.\DIRECTORY_SEPARATOR.'target'.\DIRECTORY_SEPARATOR; $this->filesystem->mirror($sourcePath, $targetPath); $this->assertTrue(is_dir($targetPath)); $this->assertFileEquals($sourcePath.'file1', $targetPath.'link1'); $this->assertTrue(is_link($targetPath.\DIRECTORY_SEPARATOR.'link1')); } public function testMirrorCopiesLinkedDirectoryContents() { $this->markAsSkippedIfSymlinkIsMissing(true); $sourcePath = $this->workspace.\DIRECTORY_SEPARATOR.'source'.\DIRECTORY_SEPARATOR; mkdir($sourcePath.'nested/', 0777, true); file_put_contents($sourcePath.'/nested/file1.txt', 'FILE1'); symlink($sourcePath.'nested', $sourcePath.'link1'); $targetPath = $this->workspace.\DIRECTORY_SEPARATOR.'target'.\DIRECTORY_SEPARATOR; $this->filesystem->mirror($sourcePath, $targetPath); $this->assertTrue(is_dir($targetPath)); $this->assertFileEquals($sourcePath.'/nested/file1.txt', $targetPath.'link1/file1.txt'); $this->assertTrue(is_link($targetPath.\DIRECTORY_SEPARATOR.'link1')); } public function testMirrorCopiesRelativeLinkedContents() { $this->markAsSkippedIfSymlinkIsMissing(true); $sourcePath = $this->workspace.\DIRECTORY_SEPARATOR.'source'.\DIRECTORY_SEPARATOR; $oldPath = getcwd(); mkdir($sourcePath.'nested/', 0777, true); file_put_contents($sourcePath.'/nested/file1.txt', 'FILE1'); chdir($sourcePath); symlink('nested', 'link1'); chdir($oldPath); $targetPath = $this->workspace.\DIRECTORY_SEPARATOR.'target'.\DIRECTORY_SEPARATOR; $this->filesystem->mirror($sourcePath, $targetPath); $this->assertTrue(is_dir($targetPath)); $this->assertFileEquals($sourcePath.'/nested/file1.txt', $targetPath.'link1/file1.txt'); $this->assertTrue(is_link($targetPath.\DIRECTORY_SEPARATOR.'link1')); $this->assertEquals('\\' === \DIRECTORY_SEPARATOR ? realpath($sourcePath.'\nested') : 'nested', readlink($targetPath.\DIRECTORY_SEPARATOR.'link1')); } public function testMirrorContentsWithSameNameAsSourceOrTargetWithoutDeleteOption() { $sourcePath = $this->workspace.\DIRECTORY_SEPARATOR.'source'.\DIRECTORY_SEPARATOR; mkdir($sourcePath); touch($sourcePath.'source'); touch($sourcePath.'target'); $targetPath = $this->workspace.\DIRECTORY_SEPARATOR.'target'.\DIRECTORY_SEPARATOR; $oldPath = getcwd(); chdir($this->workspace); $this->filesystem->mirror('source', $targetPath); chdir($oldPath); $this->assertTrue(is_dir($targetPath)); $this->assertFileExists($targetPath.'source'); $this->assertFileExists($targetPath.'target'); } public function testMirrorContentsWithSameNameAsSourceOrTargetWithDeleteOption() { $sourcePath = $this->workspace.\DIRECTORY_SEPARATOR.'source'.\DIRECTORY_SEPARATOR; mkdir($sourcePath); touch($sourcePath.'source'); $targetPath = $this->workspace.\DIRECTORY_SEPARATOR.'target'.\DIRECTORY_SEPARATOR; mkdir($targetPath); touch($targetPath.'source'); touch($targetPath.'target'); $oldPath = getcwd(); chdir($this->workspace); $this->filesystem->mirror('source', 'target', null, array('delete' => true)); chdir($oldPath); $this->assertTrue(is_dir($targetPath)); $this->assertFileExists($targetPath.'source'); $this->assertFileNotExists($targetPath.'target'); } public function testIsAbsolutePath($path, $expectedResult) { $result = $this->filesystem->isAbsolutePath($path); $this->assertEquals($expectedResult, $result); } public function providePathsForIsAbsolutePath() { return array( array('/var/lib', true), array('c:\\\\var\\lib', true), array('\\var\\lib', true), array('var/lib', false), array('../var/lib', false), array('', false), array(null, false), ); } public function testTempnam() { $dirname = $this->workspace; $filename = $this->filesystem->tempnam($dirname, 'foo'); $this->assertFileExists($filename); } public function testTempnamWithFileScheme() { $scheme = 'file://'; $dirname = $scheme.$this->workspace; $filename = $this->filesystem->tempnam($dirname, 'foo'); $this->assertStringStartsWith($scheme, $filename); $this->assertFileExists($filename); } public function testTempnamWithMockScheme() { stream_wrapper_register('mock', 'Symfony\Component\Filesystem\Tests\Fixtures\MockStream\MockStream'); $scheme = 'mock://'; $dirname = $scheme.$this->workspace; $filename = $this->filesystem->tempnam($dirname, 'foo'); $this->assertStringStartsWith($scheme, $filename); $this->assertFileExists($filename); } public function testTempnamWithZlibSchemeFails() { $scheme = 'compress.zlib://'; $dirname = $scheme.$this->workspace; $this->filesystem->tempnam($dirname, 'bar'); } public function testTempnamWithPHPTempSchemeFails() { $scheme = 'php://temp'; $dirname = $scheme; $filename = $this->filesystem->tempnam($dirname, 'bar'); $this->assertStringStartsWith($scheme, $filename); $this->assertFileNotExists($filename); } public function testTempnamWithPharSchemeFails() { if (!\Phar::canWrite()) { $this->markTestSkipped('This test cannot run when phar.readonly is 1.'); } $scheme = 'phar://'; $dirname = $scheme.$this->workspace; $pharname = 'foo.phar'; new \Phar($this->workspace.'/'.$pharname, 0, $pharname); $this->filesystem->tempnam($dirname, $pharname.'/bar'); } public function testTempnamWithHTTPSchemeFails() { $scheme = 'http://'; $dirname = $scheme.$this->workspace; $this->filesystem->tempnam($dirname, 'bar'); } public function testTempnamOnUnwritableFallsBackToSysTmp() { $scheme = 'file://'; $dirname = $scheme.$this->workspace.\DIRECTORY_SEPARATOR.'does_not_exist'; $filename = $this->filesystem->tempnam($dirname, 'bar'); $realTempDir = realpath(sys_get_temp_dir()); $this->assertStringStartsWith(rtrim($scheme.$realTempDir, \DIRECTORY_SEPARATOR), $filename); $this->assertFileExists($filename); @unlink($filename); } public function testDumpFile() { $filename = $this->workspace.\DIRECTORY_SEPARATOR.'foo'.\DIRECTORY_SEPARATOR.'baz.txt'; if ('\\' !== \DIRECTORY_SEPARATOR) { $oldMask = umask(0002); } $this->filesystem->dumpFile($filename, 'bar'); $this->assertFileExists($filename); $this->assertStringEqualsFile($filename, 'bar'); if ('\\' !== \DIRECTORY_SEPARATOR) { $this->assertFilePermissions(664, $filename); umask($oldMask); } } public function testDumpFileWithArray() { $filename = $this->workspace.\DIRECTORY_SEPARATOR.'foo'.\DIRECTORY_SEPARATOR.'baz.txt'; $this->filesystem->dumpFile($filename, array('bar')); $this->assertFileExists($filename); $this->assertStringEqualsFile($filename, 'bar'); } public function testDumpFileWithResource() { $filename = $this->workspace.\DIRECTORY_SEPARATOR.'foo'.\DIRECTORY_SEPARATOR.'baz.txt'; $resource = fopen('php://memory', 'rw'); fwrite($resource, 'bar'); fseek($resource, 0); $this->filesystem->dumpFile($filename, $resource); fclose($resource); $this->assertFileExists($filename); $this->assertStringEqualsFile($filename, 'bar'); } public function testDumpFileOverwritesAnExistingFile() { $filename = $this->workspace.\DIRECTORY_SEPARATOR.'foo.txt'; file_put_contents($filename, 'FOO BAR'); $this->filesystem->dumpFile($filename, 'bar'); $this->assertFileExists($filename); $this->assertStringEqualsFile($filename, 'bar'); } public function testDumpFileWithFileScheme() { $scheme = 'file://'; $filename = $scheme.$this->workspace.\DIRECTORY_SEPARATOR.'foo'.\DIRECTORY_SEPARATOR.'baz.txt'; $this->filesystem->dumpFile($filename, 'bar'); $this->assertFileExists($filename); $this->assertStringEqualsFile($filename, 'bar'); } public function testDumpFileWithZlibScheme() { $scheme = 'compress.zlib://'; $filename = $this->workspace.\DIRECTORY_SEPARATOR.'foo'.\DIRECTORY_SEPARATOR.'baz.txt'; $this->filesystem->dumpFile($filename, 'bar'); $this->assertFileExists(str_replace($scheme, '', $filename)); $this->assertStringEqualsFile($filename, 'bar'); } public function testAppendToFile() { $filename = $this->workspace.\DIRECTORY_SEPARATOR.'foo'.\DIRECTORY_SEPARATOR.'bar.txt'; if ('\\' !== \DIRECTORY_SEPARATOR) { $oldMask = umask(0002); } $this->filesystem->dumpFile($filename, 'foo'); $this->filesystem->appendToFile($filename, 'bar'); $this->assertFileExists($filename); $this->assertStringEqualsFile($filename, 'foobar'); if ('\\' !== \DIRECTORY_SEPARATOR) { $this->assertFilePermissions(664, $filename); umask($oldMask); } } public function testAppendToFileWithScheme() { $scheme = 'file://'; $filename = $scheme.$this->workspace.\DIRECTORY_SEPARATOR.'foo'.\DIRECTORY_SEPARATOR.'baz.txt'; $this->filesystem->dumpFile($filename, 'foo'); $this->filesystem->appendToFile($filename, 'bar'); $this->assertFileExists($filename); $this->assertStringEqualsFile($filename, 'foobar'); } public function testAppendToFileWithZlibScheme() { $scheme = 'compress.zlib://'; $filename = $this->workspace.\DIRECTORY_SEPARATOR.'foo'.\DIRECTORY_SEPARATOR.'baz.txt'; $this->filesystem->dumpFile($filename, 'foo'); $this->assertStringEqualsFile(str_replace($scheme, '', $filename), 'foo'); $this->filesystem->appendToFile($filename, 'bar'); $this->assertFileExists($filename); $this->assertStringEqualsFile($filename, 'foobar'); } public function testAppendToFileCreateTheFileIfNotExists() { $filename = $this->workspace.\DIRECTORY_SEPARATOR.'foo'.\DIRECTORY_SEPARATOR.'bar.txt'; if ('\\' !== \DIRECTORY_SEPARATOR) { $oldMask = umask(0002); } $this->filesystem->appendToFile($filename, 'bar'); if ('\\' !== \DIRECTORY_SEPARATOR) { $this->assertFilePermissions(664, $filename); umask($oldMask); } $this->assertFileExists($filename); $this->assertStringEqualsFile($filename, 'bar'); } public function testDumpKeepsExistingPermissionsWhenOverwritingAnExistingFile() { $this->markAsSkippedIfChmodIsMissing(); $filename = $this->workspace.\DIRECTORY_SEPARATOR.'foo.txt'; file_put_contents($filename, 'FOO BAR'); chmod($filename, 0745); $this->filesystem->dumpFile($filename, 'bar', null); $this->assertFilePermissions(745, $filename); } public function testCopyShouldKeepExecutionPermission() { $this->markAsSkippedIfChmodIsMissing(); $sourceFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_source_file'; $targetFilePath = $this->workspace.\DIRECTORY_SEPARATOR.'copy_target_file'; file_put_contents($sourceFilePath, 'SOURCE FILE'); chmod($sourceFilePath, 0745); $this->filesystem->copy($sourceFilePath, $targetFilePath); $this->assertFilePermissions(767, $targetFilePath); } private function normalize(string $path): string { return str_replace('/', \DIRECTORY_SEPARATOR, $path); } } 