<?php
 namespace Symfony\Component\HttpFoundation; class Cookie { protected $name; protected $value; protected $domain; protected $expire; protected $path; protected $secure; protected $httpOnly; private $raw; private $sameSite; const SAMESITE_LAX = 'lax'; const SAMESITE_STRICT = 'strict'; public static function fromString($cookie, $decode = false) { $data = array( 'expires' => 0, 'path' => '/', 'domain' => null, 'secure' => false, 'httponly' => false, 'raw' => !$decode, 'samesite' => null, ); foreach (explode(';', $cookie) as $part) { if (false === strpos($part, '=')) { $key = trim($part); $value = true; } else { list($key, $value) = explode('=', trim($part), 2); $key = trim($key); $value = trim($value); } if (!isset($data['name'])) { $data['name'] = $decode ? urldecode($key) : $key; $data['value'] = true === $value ? null : ($decode ? urldecode($value) : $value); continue; } switch ($key = strtolower($key)) { case 'name': case 'value': break; case 'max-age': $data['expires'] = time() + (int) $value; break; default: $data[$key] = $value; break; } } return new static($data['name'], $data['value'], $data['expires'], $data['path'], $data['domain'], $data['secure'], $data['httponly'], $data['raw'], $data['samesite']); } public function __construct($name, $value = null, $expire = 0, $path = '/', $domain = null, $secure = false, $httpOnly = true, $raw = false, $sameSite = null) { if (preg_match("/[=,; \t\r\n\013\014]/", $name)) { throw new \InvalidArgumentException(sprintf('The cookie name "%s" contains invalid characters.', $name)); } if (empty($name)) { throw new \InvalidArgumentException('The cookie name cannot be empty.'); } if ($expire instanceof \DateTimeInterface) { $expire = $expire->format('U'); } elseif (!is_numeric($expire)) { $expire = strtotime($expire); if (false === $expire) { throw new \InvalidArgumentException('The cookie expiration time is not valid.'); } } $this->name = $name; $this->value = $value; $this->domain = $domain; $this->expire = 0 < $expire ? (int) $expire : 0; $this->path = empty($path) ? '/' : $path; $this->secure = (bool) $secure; $this->httpOnly = (bool) $httpOnly; $this->raw = (bool) $raw; if (null !== $sameSite) { $sameSite = strtolower($sameSite); } if (!\in_array($sameSite, array(self::SAMESITE_LAX, self::SAMESITE_STRICT, null), true)) { throw new \InvalidArgumentException('The "sameSite" parameter value is not valid.'); } $this->sameSite = $sameSite; } public function __toString() { $str = ($this->isRaw() ? $this->getName() : urlencode($this->getName())).'='; if ('' === (string) $this->getValue()) { $str .= 'deleted; expires='.gmdate('D, d-M-Y H:i:s T', time() - 31536001).'; Max-Age=0'; } else { $str .= $this->isRaw() ? $this->getValue() : rawurlencode($this->getValue()); if (0 !== $this->getExpiresTime()) { $str .= '; expires='.gmdate('D, d-M-Y H:i:s T', $this->getExpiresTime()).'; Max-Age='.$this->getMaxAge(); } } if ($this->getPath()) { $str .= '; path='.$this->getPath(); } if ($this->getDomain()) { $str .= '; domain='.$this->getDomain(); } if (true === $this->isSecure()) { $str .= '; secure'; } if (true === $this->isHttpOnly()) { $str .= '; httponly'; } if (null !== $this->getSameSite()) { $str .= '; samesite='.$this->getSameSite(); } return $str; } public function getName() { return $this->name; } public function getValue() { return $this->value; } public function getDomain() { return $this->domain; } public function getExpiresTime() { return $this->expire; } public function getMaxAge() { $maxAge = $this->expire - time(); return 0 >= $maxAge ? 0 : $maxAge; } public function getPath() { return $this->path; } public function isSecure() { return $this->secure; } public function isHttpOnly() { return $this->httpOnly; } public function isCleared() { return 0 !== $this->expire && $this->expire < time(); } public function isRaw() { return $this->raw; } public function getSameSite() { return $this->sameSite; } } 