<?php
 namespace Symfony\Component\Routing\Tests\Loader; use PHPUnit\Framework\TestCase; use Symfony\Component\Config\FileLocator; use Symfony\Component\Config\Resource\FileResource; use Symfony\Component\Routing\Loader\YamlFileLoader; class YamlFileLoaderTest extends TestCase { public function testSupports() { $loader = new YamlFileLoader($this->getMockBuilder('Symfony\Component\Config\FileLocator')->getMock()); $this->assertTrue($loader->supports('foo.yml'), '->supports() returns true if the resource is loadable'); $this->assertTrue($loader->supports('foo.yaml'), '->supports() returns true if the resource is loadable'); $this->assertFalse($loader->supports('foo.foo'), '->supports() returns true if the resource is loadable'); $this->assertTrue($loader->supports('foo.yml', 'yaml'), '->supports() checks the resource type if specified'); $this->assertTrue($loader->supports('foo.yaml', 'yaml'), '->supports() checks the resource type if specified'); $this->assertFalse($loader->supports('foo.yml', 'foo'), '->supports() checks the resource type if specified'); } public function testLoadDoesNothingIfEmpty() { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures'))); $collection = $loader->load('empty.yml'); $this->assertEquals(array(), $collection->all()); $this->assertEquals(array(new FileResource(realpath(__DIR__.'/../Fixtures/empty.yml'))), $collection->getResources()); } public function testLoadThrowsExceptionWithInvalidFile($filePath) { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures'))); $loader->load($filePath); } public function getPathsToInvalidFiles() { return array( array('nonvalid.yml'), array('nonvalid2.yml'), array('incomplete.yml'), array('nonvalidkeys.yml'), array('nonesense_resource_plus_path.yml'), array('nonesense_type_without_resource.yml'), array('bad_format.yml'), ); } public function testLoadSpecialRouteName() { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures'))); $routeCollection = $loader->load('special_route_name.yml'); $route = $routeCollection->get('#$péß^a|'); $this->assertInstanceOf('Symfony\Component\Routing\Route', $route); $this->assertSame('/true', $route->getPath()); } public function testLoadWithRoute() { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures'))); $routeCollection = $loader->load('validpattern.yml'); $route = $routeCollection->get('blog_show'); $this->assertInstanceOf('Symfony\Component\Routing\Route', $route); $this->assertSame('/blog/{slug}', $route->getPath()); $this->assertSame('{locale}.example.com', $route->getHost()); $this->assertSame('MyBundle:Blog:show', $route->getDefault('_controller')); $this->assertSame('\w+', $route->getRequirement('locale')); $this->assertSame('RouteCompiler', $route->getOption('compiler_class')); $this->assertEquals(array('GET', 'POST', 'PUT', 'OPTIONS'), $route->getMethods()); $this->assertEquals(array('https'), $route->getSchemes()); $this->assertEquals('context.getMethod() == "GET"', $route->getCondition()); } public function testLoadWithResource() { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures'))); $routeCollection = $loader->load('validresource.yml'); $routes = $routeCollection->all(); $this->assertCount(2, $routes, 'Two routes are loaded'); $this->assertContainsOnly('Symfony\Component\Routing\Route', $routes); foreach ($routes as $route) { $this->assertSame('/{foo}/blog/{slug}', $route->getPath()); $this->assertSame('123', $route->getDefault('foo')); $this->assertSame('\d+', $route->getRequirement('foo')); $this->assertSame('bar', $route->getOption('foo')); $this->assertSame('', $route->getHost()); $this->assertSame('context.getMethod() == "POST"', $route->getCondition()); } } public function testLoadRouteWithControllerAttribute() { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures/controller'))); $routeCollection = $loader->load('routing.yml'); $route = $routeCollection->get('app_homepage'); $this->assertSame('AppBundle:Homepage:show', $route->getDefault('_controller')); } public function testLoadRouteWithoutControllerAttribute() { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures/controller'))); $routeCollection = $loader->load('routing.yml'); $route = $routeCollection->get('app_logout'); $this->assertNull($route->getDefault('_controller')); } public function testLoadRouteWithControllerSetInDefaults() { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures/controller'))); $routeCollection = $loader->load('routing.yml'); $route = $routeCollection->get('app_blog'); $this->assertSame('AppBundle:Blog:list', $route->getDefault('_controller')); } public function testOverrideControllerInDefaults() { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures/controller'))); $loader->load('override_defaults.yml'); } public function testImportRouteWithController($file) { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures/controller'))); $routeCollection = $loader->load($file); $route = $routeCollection->get('app_homepage'); $this->assertSame('FrameworkBundle:Template:template', $route->getDefault('_controller')); $route = $routeCollection->get('app_blog'); $this->assertSame('FrameworkBundle:Template:template', $route->getDefault('_controller')); $route = $routeCollection->get('app_logout'); $this->assertSame('FrameworkBundle:Template:template', $route->getDefault('_controller')); } public function provideFilesImportingRoutesWithControllers() { yield array('import_controller.yml'); yield array('import__controller.yml'); } public function testImportWithOverriddenController() { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures/controller'))); $loader->load('import_override_defaults.yml'); } public function testImportRouteWithGlobMatchingSingleFile() { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures/glob'))); $routeCollection = $loader->load('import_single.yml'); $route = $routeCollection->get('bar_route'); $this->assertSame('AppBundle:Bar:view', $route->getDefault('_controller')); } public function testImportRouteWithGlobMatchingMultipleFiles() { $loader = new YamlFileLoader(new FileLocator(array(__DIR__.'/../Fixtures/glob'))); $routeCollection = $loader->load('import_multiple.yml'); $route = $routeCollection->get('bar_route'); $this->assertSame('AppBundle:Bar:view', $route->getDefault('_controller')); $route = $routeCollection->get('baz_route'); $this->assertSame('AppBundle:Baz:view', $route->getDefault('_controller')); } } 